# Copyright (c) 2021 AccelByte Inc. All Rights Reserved.
# This is licensed software from AccelByte Inc, for limitations
# and restrictions contact your company contract manager.
#
# Code generated. DO NOT EDIT!

# template file: ags_py_codegen

# pylint: disable=duplicate-code
# pylint: disable=line-too-long
# pylint: disable=missing-function-docstring
# pylint: disable=missing-function-docstring
# pylint: disable=missing-module-docstring
# pylint: disable=too-many-arguments
# pylint: disable=too-many-branches
# pylint: disable=too-many-instance-attributes
# pylint: disable=too-many-lines
# pylint: disable=too-many-locals
# pylint: disable=too-many-public-methods
# pylint: disable=too-many-return-statements
# pylint: disable=too-many-statements
# pylint: disable=unused-import

from typing import Any, Dict, List, Optional, Tuple, Union

from ....core import HeaderStr
from ....core import get_namespace as get_services_namespace
from ....core import run_request
from ....core import run_request_async
from ....core import same_doc_as

from ..models import CreateLocalizedPolicyVersionRequest
from ..models import CreateLocalizedPolicyVersionResponse
from ..models import ErrorEntity
from ..models import RetrieveLocalizedPolicyVersionPublicResponse
from ..models import RetrieveLocalizedPolicyVersionResponse
from ..models import UpdateLocalizedPolicyVersionRequest
from ..models import UpdateLocalizedPolicyVersionResponse
from ..models import UploadLocalizedPolicyVersionAttachmentResponse
from ..models import UploadPolicyVersionAttachmentRequest

from ..operations.localized_policy_versions_with_namespace import (
    CreateLocalizedPolicyVersion,
)
from ..operations.localized_policy_versions_with_namespace import DeleteLocalizedPolicy
from ..operations.localized_policy_versions_with_namespace import (
    PublicRetrieveSingleLocalizedPolicyVersion,
)
from ..operations.localized_policy_versions_with_namespace import RequestPresignedURL
from ..operations.localized_policy_versions_with_namespace import (
    RetrieveLocalizedPolicyVersions,
)
from ..operations.localized_policy_versions_with_namespace import (
    RetrieveSingleLocalizedPolicyVersion,
)
from ..operations.localized_policy_versions_with_namespace import (
    SetDefaultLocalizedPolicy,
)
from ..operations.localized_policy_versions_with_namespace import (
    UpdateLocalizedPolicyVersion,
)


@same_doc_as(CreateLocalizedPolicyVersion)
def create_localized_policy_version(
    policy_version_id: str,
    body: Optional[CreateLocalizedPolicyVersionRequest] = None,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Create a Localized Version from Country-Specific Policy (createLocalizedPolicyVersion)

    Create a version of a particular country-specific policy.

    Properties:
        url: /agreement/admin/namespaces/{namespace}/localized-policy-versions/versions/{policyVersionId}

        method: POST

        tags: ["Localized Policy Versions With Namespace"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        body: (body) OPTIONAL CreateLocalizedPolicyVersionRequest in body

        namespace: (namespace) REQUIRED str in path

        policy_version_id: (policyVersionId) REQUIRED str in path

    Responses:
        201: Created - CreateLocalizedPolicyVersionResponse (successful operation)

        400: Bad Request - ErrorEntity (40033: errors.net.accelbyte.platform.legal.invalid_policy_version)

        409: Conflict - ErrorEntity (40044: errors.net.accelbyte.platform.legal.localized_policy_version_already_exist)
    """
    if namespace is None:
        namespace, error = get_services_namespace(sdk=kwargs.get("sdk"))
        if error:
            return None, error
    request = CreateLocalizedPolicyVersion.create(
        policy_version_id=policy_version_id,
        body=body,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(CreateLocalizedPolicyVersion)
async def create_localized_policy_version_async(
    policy_version_id: str,
    body: Optional[CreateLocalizedPolicyVersionRequest] = None,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Create a Localized Version from Country-Specific Policy (createLocalizedPolicyVersion)

    Create a version of a particular country-specific policy.

    Properties:
        url: /agreement/admin/namespaces/{namespace}/localized-policy-versions/versions/{policyVersionId}

        method: POST

        tags: ["Localized Policy Versions With Namespace"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        body: (body) OPTIONAL CreateLocalizedPolicyVersionRequest in body

        namespace: (namespace) REQUIRED str in path

        policy_version_id: (policyVersionId) REQUIRED str in path

    Responses:
        201: Created - CreateLocalizedPolicyVersionResponse (successful operation)

        400: Bad Request - ErrorEntity (40033: errors.net.accelbyte.platform.legal.invalid_policy_version)

        409: Conflict - ErrorEntity (40044: errors.net.accelbyte.platform.legal.localized_policy_version_already_exist)
    """
    if namespace is None:
        namespace, error = get_services_namespace(sdk=kwargs.get("sdk"))
        if error:
            return None, error
    request = CreateLocalizedPolicyVersion.create(
        policy_version_id=policy_version_id,
        body=body,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@same_doc_as(DeleteLocalizedPolicy)
def delete_localized_policy(
    localized_policy_version_id: str,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Delete Localized Policy (deleteLocalizedPolicy)

    Delete localized version policy.
    Can only be deleted if match these criteria:


      * Policy Version that this localized policy version belongs to is not active
      * Has never been accepted by any user

    Properties:
        url: /agreement/admin/namespaces/{namespace}/localized-policy-versions/versions/{localizedPolicyVersionId}

        method: DELETE

        tags: ["Localized Policy Versions With Namespace"]

        consumes: []

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        localized_policy_version_id: (localizedPolicyVersionId) REQUIRED str in path

        namespace: (namespace) REQUIRED str in path

    Responses:
        204: No Content - (successful operation)

        400: Bad Request - ErrorEntity (40075: errors.net.accelbyte.platform.legal.localized_policy_version_is_published_and_already_accepted_by_user | 40076: errors.net.accelbyte.platform.legal.localized_policy_version_is_active)
    """
    if namespace is None:
        namespace, error = get_services_namespace(sdk=kwargs.get("sdk"))
        if error:
            return None, error
    request = DeleteLocalizedPolicy.create(
        localized_policy_version_id=localized_policy_version_id,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(DeleteLocalizedPolicy)
async def delete_localized_policy_async(
    localized_policy_version_id: str,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Delete Localized Policy (deleteLocalizedPolicy)

    Delete localized version policy.
    Can only be deleted if match these criteria:


      * Policy Version that this localized policy version belongs to is not active
      * Has never been accepted by any user

    Properties:
        url: /agreement/admin/namespaces/{namespace}/localized-policy-versions/versions/{localizedPolicyVersionId}

        method: DELETE

        tags: ["Localized Policy Versions With Namespace"]

        consumes: []

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        localized_policy_version_id: (localizedPolicyVersionId) REQUIRED str in path

        namespace: (namespace) REQUIRED str in path

    Responses:
        204: No Content - (successful operation)

        400: Bad Request - ErrorEntity (40075: errors.net.accelbyte.platform.legal.localized_policy_version_is_published_and_already_accepted_by_user | 40076: errors.net.accelbyte.platform.legal.localized_policy_version_is_active)
    """
    if namespace is None:
        namespace, error = get_services_namespace(sdk=kwargs.get("sdk"))
        if error:
            return None, error
    request = DeleteLocalizedPolicy.create(
        localized_policy_version_id=localized_policy_version_id,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@same_doc_as(PublicRetrieveSingleLocalizedPolicyVersion)
def public_retrieve_single_localized_policy_version(
    localized_policy_version_id: str,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Retrieve a Localized Version (publicRetrieveSingleLocalizedPolicyVersion)

    Retrieve specific localized policy version including the policy version and base policy version where the localized policy version located.
    Other detail info:

    Properties:
        url: /agreement/public/namespaces/{namespace}/localized-policy-versions/{localizedPolicyVersionId}

        method: GET

        tags: ["Localized Policy Versions With Namespace"]

        consumes: []

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        localized_policy_version_id: (localizedPolicyVersionId) REQUIRED str in path

        namespace: (namespace) REQUIRED str in path

    Responses:
        200: OK - RetrieveLocalizedPolicyVersionPublicResponse (successful operation)

        403: Forbidden - ErrorEntity (40081: errors.net.accelbyte.platform.legal.policy_not_accessible)

        404: Not Found - ErrorEntity (40038: errors.net.accelbyte.platform.legal.localized_policy_version_not_found)
    """
    if namespace is None:
        namespace, error = get_services_namespace(sdk=kwargs.get("sdk"))
        if error:
            return None, error
    request = PublicRetrieveSingleLocalizedPolicyVersion.create(
        localized_policy_version_id=localized_policy_version_id,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(PublicRetrieveSingleLocalizedPolicyVersion)
async def public_retrieve_single_localized_policy_version_async(
    localized_policy_version_id: str,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Retrieve a Localized Version (publicRetrieveSingleLocalizedPolicyVersion)

    Retrieve specific localized policy version including the policy version and base policy version where the localized policy version located.
    Other detail info:

    Properties:
        url: /agreement/public/namespaces/{namespace}/localized-policy-versions/{localizedPolicyVersionId}

        method: GET

        tags: ["Localized Policy Versions With Namespace"]

        consumes: []

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        localized_policy_version_id: (localizedPolicyVersionId) REQUIRED str in path

        namespace: (namespace) REQUIRED str in path

    Responses:
        200: OK - RetrieveLocalizedPolicyVersionPublicResponse (successful operation)

        403: Forbidden - ErrorEntity (40081: errors.net.accelbyte.platform.legal.policy_not_accessible)

        404: Not Found - ErrorEntity (40038: errors.net.accelbyte.platform.legal.localized_policy_version_not_found)
    """
    if namespace is None:
        namespace, error = get_services_namespace(sdk=kwargs.get("sdk"))
        if error:
            return None, error
    request = PublicRetrieveSingleLocalizedPolicyVersion.create(
        localized_policy_version_id=localized_policy_version_id,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@same_doc_as(RequestPresignedURL)
def request_presigned_url(
    localized_policy_version_id: str,
    body: Optional[UploadPolicyVersionAttachmentRequest] = None,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Request Presigned URL for Upload Document (requestPresignedURL)

    Request presigned URL for upload attachment for a particular localized version of base policy.

    Properties:
        url: /agreement/admin/namespaces/{namespace}/localized-policy-versions/{localizedPolicyVersionId}/attachments

        method: POST

        tags: ["Localized Policy Versions With Namespace"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        body: (body) OPTIONAL UploadPolicyVersionAttachmentRequest in body

        localized_policy_version_id: (localizedPolicyVersionId) REQUIRED str in path

        namespace: (namespace) REQUIRED str in path

    Responses:
        201: Created - UploadLocalizedPolicyVersionAttachmentResponse (successful operation)

        400: Bad Request - ErrorEntity (40034: errors.net.accelbyte.platform.legal.invalid_file_type | 40037: errors.net.accelbyte.platform.legal.invalid_localized_policy_version | 40042: errors.net.accelbyte.platform.legal.policy_version_freezed)
    """
    if namespace is None:
        namespace, error = get_services_namespace(sdk=kwargs.get("sdk"))
        if error:
            return None, error
    request = RequestPresignedURL.create(
        localized_policy_version_id=localized_policy_version_id,
        body=body,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(RequestPresignedURL)
async def request_presigned_url_async(
    localized_policy_version_id: str,
    body: Optional[UploadPolicyVersionAttachmentRequest] = None,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Request Presigned URL for Upload Document (requestPresignedURL)

    Request presigned URL for upload attachment for a particular localized version of base policy.

    Properties:
        url: /agreement/admin/namespaces/{namespace}/localized-policy-versions/{localizedPolicyVersionId}/attachments

        method: POST

        tags: ["Localized Policy Versions With Namespace"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        body: (body) OPTIONAL UploadPolicyVersionAttachmentRequest in body

        localized_policy_version_id: (localizedPolicyVersionId) REQUIRED str in path

        namespace: (namespace) REQUIRED str in path

    Responses:
        201: Created - UploadLocalizedPolicyVersionAttachmentResponse (successful operation)

        400: Bad Request - ErrorEntity (40034: errors.net.accelbyte.platform.legal.invalid_file_type | 40037: errors.net.accelbyte.platform.legal.invalid_localized_policy_version | 40042: errors.net.accelbyte.platform.legal.policy_version_freezed)
    """
    if namespace is None:
        namespace, error = get_services_namespace(sdk=kwargs.get("sdk"))
        if error:
            return None, error
    request = RequestPresignedURL.create(
        localized_policy_version_id=localized_policy_version_id,
        body=body,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@same_doc_as(RetrieveLocalizedPolicyVersions)
def retrieve_localized_policy_versions(
    policy_version_id: str,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Retrieve Versions from Country-Specific Policy (retrieveLocalizedPolicyVersions)

    Retrieve versions of a particular country-specific policy.

    Properties:
        url: /agreement/admin/namespaces/{namespace}/localized-policy-versions/versions/{policyVersionId}

        method: GET

        tags: ["Localized Policy Versions With Namespace"]

        consumes: []

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        namespace: (namespace) REQUIRED str in path

        policy_version_id: (policyVersionId) REQUIRED str in path

    Responses:
        200: OK - List[RetrieveLocalizedPolicyVersionResponse] (successful operation)
    """
    if namespace is None:
        namespace, error = get_services_namespace(sdk=kwargs.get("sdk"))
        if error:
            return None, error
    request = RetrieveLocalizedPolicyVersions.create(
        policy_version_id=policy_version_id,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(RetrieveLocalizedPolicyVersions)
async def retrieve_localized_policy_versions_async(
    policy_version_id: str,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Retrieve Versions from Country-Specific Policy (retrieveLocalizedPolicyVersions)

    Retrieve versions of a particular country-specific policy.

    Properties:
        url: /agreement/admin/namespaces/{namespace}/localized-policy-versions/versions/{policyVersionId}

        method: GET

        tags: ["Localized Policy Versions With Namespace"]

        consumes: []

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        namespace: (namespace) REQUIRED str in path

        policy_version_id: (policyVersionId) REQUIRED str in path

    Responses:
        200: OK - List[RetrieveLocalizedPolicyVersionResponse] (successful operation)
    """
    if namespace is None:
        namespace, error = get_services_namespace(sdk=kwargs.get("sdk"))
        if error:
            return None, error
    request = RetrieveLocalizedPolicyVersions.create(
        policy_version_id=policy_version_id,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@same_doc_as(RetrieveSingleLocalizedPolicyVersion)
def retrieve_single_localized_policy_version(
    localized_policy_version_id: str,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Retrieve a Localized Version from Country-Specific Policy (retrieveSingleLocalizedPolicyVersion)

    Retrieve a version of a particular country-specific policy.

    Properties:
        url: /agreement/admin/namespaces/{namespace}/localized-policy-versions/{localizedPolicyVersionId}

        method: GET

        tags: ["Localized Policy Versions With Namespace"]

        consumes: []

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        localized_policy_version_id: (localizedPolicyVersionId) REQUIRED str in path

        namespace: (namespace) REQUIRED str in path

    Responses:
        200: OK - RetrieveLocalizedPolicyVersionResponse (successful operation)

        404: Not Found - ErrorEntity (2912: errors.net.accelbyte.platform.legal.policy_version_not_found)
    """
    if namespace is None:
        namespace, error = get_services_namespace(sdk=kwargs.get("sdk"))
        if error:
            return None, error
    request = RetrieveSingleLocalizedPolicyVersion.create(
        localized_policy_version_id=localized_policy_version_id,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(RetrieveSingleLocalizedPolicyVersion)
async def retrieve_single_localized_policy_version_async(
    localized_policy_version_id: str,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Retrieve a Localized Version from Country-Specific Policy (retrieveSingleLocalizedPolicyVersion)

    Retrieve a version of a particular country-specific policy.

    Properties:
        url: /agreement/admin/namespaces/{namespace}/localized-policy-versions/{localizedPolicyVersionId}

        method: GET

        tags: ["Localized Policy Versions With Namespace"]

        consumes: []

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        localized_policy_version_id: (localizedPolicyVersionId) REQUIRED str in path

        namespace: (namespace) REQUIRED str in path

    Responses:
        200: OK - RetrieveLocalizedPolicyVersionResponse (successful operation)

        404: Not Found - ErrorEntity (2912: errors.net.accelbyte.platform.legal.policy_version_not_found)
    """
    if namespace is None:
        namespace, error = get_services_namespace(sdk=kwargs.get("sdk"))
        if error:
            return None, error
    request = RetrieveSingleLocalizedPolicyVersion.create(
        localized_policy_version_id=localized_policy_version_id,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@same_doc_as(SetDefaultLocalizedPolicy)
def set_default_localized_policy(
    localized_policy_version_id: str,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Set Default Localized Policy (setDefaultLocalizedPolicy)

    Update a localized version policy to be the default.

    Properties:
        url: /agreement/admin/namespaces/{namespace}/localized-policy-versions/{localizedPolicyVersionId}/default

        method: PATCH

        tags: ["Localized Policy Versions With Namespace"]

        consumes: []

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        localized_policy_version_id: (localizedPolicyVersionId) REQUIRED str in path

        namespace: (namespace) REQUIRED str in path

    Responses:
        200: OK - (successful operation)
    """
    if namespace is None:
        namespace, error = get_services_namespace(sdk=kwargs.get("sdk"))
        if error:
            return None, error
    request = SetDefaultLocalizedPolicy.create(
        localized_policy_version_id=localized_policy_version_id,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(SetDefaultLocalizedPolicy)
async def set_default_localized_policy_async(
    localized_policy_version_id: str,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Set Default Localized Policy (setDefaultLocalizedPolicy)

    Update a localized version policy to be the default.

    Properties:
        url: /agreement/admin/namespaces/{namespace}/localized-policy-versions/{localizedPolicyVersionId}/default

        method: PATCH

        tags: ["Localized Policy Versions With Namespace"]

        consumes: []

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        localized_policy_version_id: (localizedPolicyVersionId) REQUIRED str in path

        namespace: (namespace) REQUIRED str in path

    Responses:
        200: OK - (successful operation)
    """
    if namespace is None:
        namespace, error = get_services_namespace(sdk=kwargs.get("sdk"))
        if error:
            return None, error
    request = SetDefaultLocalizedPolicy.create(
        localized_policy_version_id=localized_policy_version_id,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@same_doc_as(UpdateLocalizedPolicyVersion)
def update_localized_policy_version(
    localized_policy_version_id: str,
    body: Optional[UpdateLocalizedPolicyVersionRequest] = None,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Update a Localized Version from Country-Specific Policy (updateLocalizedPolicyVersion)

    Update a version of a particular country-specific policy.

    Properties:
        url: /agreement/admin/namespaces/{namespace}/localized-policy-versions/{localizedPolicyVersionId}

        method: PUT

        tags: ["Localized Policy Versions With Namespace"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        body: (body) OPTIONAL UpdateLocalizedPolicyVersionRequest in body

        localized_policy_version_id: (localizedPolicyVersionId) REQUIRED str in path

        namespace: (namespace) REQUIRED str in path

    Responses:
        200: OK - UpdateLocalizedPolicyVersionResponse (successful operation)

        400: Bad Request - ErrorEntity (40035: errors.net.accelbyte.platform.legal.invalid_policy_version)
    """
    if namespace is None:
        namespace, error = get_services_namespace(sdk=kwargs.get("sdk"))
        if error:
            return None, error
    request = UpdateLocalizedPolicyVersion.create(
        localized_policy_version_id=localized_policy_version_id,
        body=body,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(UpdateLocalizedPolicyVersion)
async def update_localized_policy_version_async(
    localized_policy_version_id: str,
    body: Optional[UpdateLocalizedPolicyVersionRequest] = None,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Update a Localized Version from Country-Specific Policy (updateLocalizedPolicyVersion)

    Update a version of a particular country-specific policy.

    Properties:
        url: /agreement/admin/namespaces/{namespace}/localized-policy-versions/{localizedPolicyVersionId}

        method: PUT

        tags: ["Localized Policy Versions With Namespace"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        body: (body) OPTIONAL UpdateLocalizedPolicyVersionRequest in body

        localized_policy_version_id: (localizedPolicyVersionId) REQUIRED str in path

        namespace: (namespace) REQUIRED str in path

    Responses:
        200: OK - UpdateLocalizedPolicyVersionResponse (successful operation)

        400: Bad Request - ErrorEntity (40035: errors.net.accelbyte.platform.legal.invalid_policy_version)
    """
    if namespace is None:
        namespace, error = get_services_namespace(sdk=kwargs.get("sdk"))
        if error:
            return None, error
    request = UpdateLocalizedPolicyVersion.create(
        localized_policy_version_id=localized_policy_version_id,
        body=body,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )



# Copyright (c) 2021 AccelByte Inc. All Rights Reserved.
# This is licensed software from AccelByte Inc, for limitations
# and restrictions contact your company contract manager.
#
# Code generated. DO NOT EDIT!

# template file: ags_py_codegen

# AccelByte Gaming Services Platform Service

# pylint: disable=duplicate-code
# pylint: disable=line-too-long
# pylint: disable=missing-function-docstring
# pylint: disable=missing-module-docstring
# pylint: disable=too-many-arguments
# pylint: disable=too-many-branches
# pylint: disable=too-many-instance-attributes
# pylint: disable=too-many-lines
# pylint: disable=too-many-locals
# pylint: disable=too-many-public-methods
# pylint: disable=too-many-return-statements
# pylint: disable=too-many-statements
# pylint: disable=unused-import

from __future__ import annotations
from typing import Any, Dict, List, Optional, Tuple, Union

from ....core import Model
from ....core import StrEnum

from ..models.simple_entitlement import SimpleEntitlement
from ..models.simple_user_dlc import SimpleUserDLC
from ..models.simple_wallet import SimpleWallet


class ProgressionEnum(StrEnum):
    IN_PROGRESS = "IN_PROGRESS"
    SUCCESS = "SUCCESS"


class PlatformAccountClosureHistoryInfo(Model):
    """Platform account closure history info (PlatformAccountClosureHistoryInfo)

    Properties:
        id_: (id) REQUIRED str

        namespace: (namespace) REQUIRED str

        platform: (platform) REQUIRED str

        progression: (progression) REQUIRED Union[str, ProgressionEnum]

        user_id: (userId) REQUIRED str

        entitlements: (entitlements) OPTIONAL List[SimpleEntitlement]

        user_dlc_s: (userDlcS) OPTIONAL List[SimpleUserDLC]

        wallets: (wallets) OPTIONAL List[SimpleWallet]
    """

    # region fields

    id_: str  # REQUIRED
    namespace: str  # REQUIRED
    platform: str  # REQUIRED
    progression: Union[str, ProgressionEnum]  # REQUIRED
    user_id: str  # REQUIRED
    entitlements: List[SimpleEntitlement]  # OPTIONAL
    user_dlc_s: List[SimpleUserDLC]  # OPTIONAL
    wallets: List[SimpleWallet]  # OPTIONAL

    # endregion fields

    # region with_x methods

    def with_id(self, value: str) -> PlatformAccountClosureHistoryInfo:
        self.id_ = value
        return self

    def with_namespace(self, value: str) -> PlatformAccountClosureHistoryInfo:
        self.namespace = value
        return self

    def with_platform(self, value: str) -> PlatformAccountClosureHistoryInfo:
        self.platform = value
        return self

    def with_progression(
        self, value: Union[str, ProgressionEnum]
    ) -> PlatformAccountClosureHistoryInfo:
        self.progression = value
        return self

    def with_user_id(self, value: str) -> PlatformAccountClosureHistoryInfo:
        self.user_id = value
        return self

    def with_entitlements(
        self, value: List[SimpleEntitlement]
    ) -> PlatformAccountClosureHistoryInfo:
        self.entitlements = value
        return self

    def with_user_dlc_s(
        self, value: List[SimpleUserDLC]
    ) -> PlatformAccountClosureHistoryInfo:
        self.user_dlc_s = value
        return self

    def with_wallets(
        self, value: List[SimpleWallet]
    ) -> PlatformAccountClosureHistoryInfo:
        self.wallets = value
        return self

    # endregion with_x methods

    # region to methods

    def to_dict(self, include_empty: bool = False) -> dict:
        result: dict = {}
        if hasattr(self, "id_"):
            result["id"] = str(self.id_)
        elif include_empty:
            result["id"] = ""
        if hasattr(self, "namespace"):
            result["namespace"] = str(self.namespace)
        elif include_empty:
            result["namespace"] = ""
        if hasattr(self, "platform"):
            result["platform"] = str(self.platform)
        elif include_empty:
            result["platform"] = ""
        if hasattr(self, "progression"):
            result["progression"] = str(self.progression)
        elif include_empty:
            result["progression"] = Union[str, ProgressionEnum]()
        if hasattr(self, "user_id"):
            result["userId"] = str(self.user_id)
        elif include_empty:
            result["userId"] = ""
        if hasattr(self, "entitlements"):
            result["entitlements"] = [
                i0.to_dict(include_empty=include_empty) for i0 in self.entitlements
            ]
        elif include_empty:
            result["entitlements"] = []
        if hasattr(self, "user_dlc_s"):
            result["userDlcS"] = [
                i0.to_dict(include_empty=include_empty) for i0 in self.user_dlc_s
            ]
        elif include_empty:
            result["userDlcS"] = []
        if hasattr(self, "wallets"):
            result["wallets"] = [
                i0.to_dict(include_empty=include_empty) for i0 in self.wallets
            ]
        elif include_empty:
            result["wallets"] = []
        return result

    # endregion to methods

    # region static methods

    @classmethod
    def create(
        cls,
        id_: str,
        namespace: str,
        platform: str,
        progression: Union[str, ProgressionEnum],
        user_id: str,
        entitlements: Optional[List[SimpleEntitlement]] = None,
        user_dlc_s: Optional[List[SimpleUserDLC]] = None,
        wallets: Optional[List[SimpleWallet]] = None,
        **kwargs,
    ) -> PlatformAccountClosureHistoryInfo:
        instance = cls()
        instance.id_ = id_
        instance.namespace = namespace
        instance.platform = platform
        instance.progression = progression
        instance.user_id = user_id
        if entitlements is not None:
            instance.entitlements = entitlements
        if user_dlc_s is not None:
            instance.user_dlc_s = user_dlc_s
        if wallets is not None:
            instance.wallets = wallets
        return instance

    @classmethod
    def create_from_dict(
        cls, dict_: dict, include_empty: bool = False
    ) -> PlatformAccountClosureHistoryInfo:
        instance = cls()
        if not dict_:
            return instance
        if "id" in dict_ and dict_["id"] is not None:
            instance.id_ = str(dict_["id"])
        elif include_empty:
            instance.id_ = ""
        if "namespace" in dict_ and dict_["namespace"] is not None:
            instance.namespace = str(dict_["namespace"])
        elif include_empty:
            instance.namespace = ""
        if "platform" in dict_ and dict_["platform"] is not None:
            instance.platform = str(dict_["platform"])
        elif include_empty:
            instance.platform = ""
        if "progression" in dict_ and dict_["progression"] is not None:
            instance.progression = str(dict_["progression"])
        elif include_empty:
            instance.progression = Union[str, ProgressionEnum]()
        if "userId" in dict_ and dict_["userId"] is not None:
            instance.user_id = str(dict_["userId"])
        elif include_empty:
            instance.user_id = ""
        if "entitlements" in dict_ and dict_["entitlements"] is not None:
            instance.entitlements = [
                SimpleEntitlement.create_from_dict(i0, include_empty=include_empty)
                for i0 in dict_["entitlements"]
            ]
        elif include_empty:
            instance.entitlements = []
        if "userDlcS" in dict_ and dict_["userDlcS"] is not None:
            instance.user_dlc_s = [
                SimpleUserDLC.create_from_dict(i0, include_empty=include_empty)
                for i0 in dict_["userDlcS"]
            ]
        elif include_empty:
            instance.user_dlc_s = []
        if "wallets" in dict_ and dict_["wallets"] is not None:
            instance.wallets = [
                SimpleWallet.create_from_dict(i0, include_empty=include_empty)
                for i0 in dict_["wallets"]
            ]
        elif include_empty:
            instance.wallets = []
        return instance

    @classmethod
    def create_many_from_dict(
        cls, dict_: dict, include_empty: bool = False
    ) -> Dict[str, PlatformAccountClosureHistoryInfo]:
        return (
            {k: cls.create_from_dict(v, include_empty=include_empty) for k, v in dict_}
            if dict_
            else {}
        )

    @classmethod
    def create_many_from_list(
        cls, list_: list, include_empty: bool = False
    ) -> List[PlatformAccountClosureHistoryInfo]:
        return (
            [cls.create_from_dict(i, include_empty=include_empty) for i in list_]
            if list_
            else []
        )

    @classmethod
    def create_from_any(
        cls, any_: any, include_empty: bool = False, many: bool = False
    ) -> Union[
        PlatformAccountClosureHistoryInfo,
        List[PlatformAccountClosureHistoryInfo],
        Dict[Any, PlatformAccountClosureHistoryInfo],
    ]:
        if many:
            if isinstance(any_, dict):
                return cls.create_many_from_dict(any_, include_empty=include_empty)
            elif isinstance(any_, list):
                return cls.create_many_from_list(any_, include_empty=include_empty)
            else:
                raise ValueError()
        else:
            return cls.create_from_dict(any_, include_empty=include_empty)

    @staticmethod
    def get_field_info() -> Dict[str, str]:
        return {
            "id": "id_",
            "namespace": "namespace",
            "platform": "platform",
            "progression": "progression",
            "userId": "user_id",
            "entitlements": "entitlements",
            "userDlcS": "user_dlc_s",
            "wallets": "wallets",
        }

    @staticmethod
    def get_required_map() -> Dict[str, bool]:
        return {
            "id": True,
            "namespace": True,
            "platform": True,
            "progression": True,
            "userId": True,
            "entitlements": False,
            "userDlcS": False,
            "wallets": False,
        }

    @staticmethod
    def get_enum_map() -> Dict[str, List[Any]]:
        return {
            "progression": ["IN_PROGRESS", "SUCCESS"],
        }

    # endregion static methods



# Copyright (c) 2021 AccelByte Inc. All Rights Reserved.
# This is licensed software from AccelByte Inc, for limitations
# and restrictions contact your company contract manager.
#
# Code generated. DO NOT EDIT!

# template file: ags_py_codegen

"""Auto-generated package that contains models used by the AccelByte Gaming Services Leaderboard Service."""

__version__ = "2.36.1"
__author__ = "AccelByte"
__email__ = "dev@accelbyte.net"

# pylint: disable=line-too-long

from .create_leaderboard_conf_2ad364 import CreateLeaderboardConfigurationAdminV3
from .delete_bulk_leaderboard_f7002d import DeleteBulkLeaderboardConfigurationAdminV3
from .delete_leaderboard_conf_8f143c import DeleteLeaderboardConfigurationAdminV3
from .get_leaderboard_configu_33150a import GetLeaderboardConfigurationAdminV3
from .get_leaderboard_configu_290af8 import GetLeaderboardConfigurationPublicV3
from .get_leaderboard_configu_9b2b53 import GetLeaderboardConfigurationsAdminV3
from .get_leaderboard_configu_b16dd3 import GetLeaderboardConfigurationsPublicV3
from .hard_delete_leaderboard_c58ff0 import HardDeleteLeaderboardAdminV3
from .update_leaderboard_conf_1b8e6b import UpdateLeaderboardConfigurationAdminV3



# Copyright (c) 2021 AccelByte Inc. All Rights Reserved.
# This is licensed software from AccelByte Inc, for limitations
# and restrictions contact your company contract manager.
#
# Code generated. DO NOT EDIT!

# template file: ags_py_codegen

# Custom Service Manager

# pylint: disable=duplicate-code
# pylint: disable=line-too-long
# pylint: disable=missing-function-docstring
# pylint: disable=missing-module-docstring
# pylint: disable=too-many-arguments
# pylint: disable=too-many-branches
# pylint: disable=too-many-instance-attributes
# pylint: disable=too-many-lines
# pylint: disable=too-many-locals
# pylint: disable=too-many-public-methods
# pylint: disable=too-many-return-statements
# pylint: disable=too-many-statements
# pylint: disable=unused-import

from __future__ import annotations
from typing import Any, Dict, List, Optional, Tuple, Union

from ....core import Model


class ApimodelCreateKeyValueCredentialResponse(Model):
    """Apimodel create key value credential response (apimodel.CreateKeyValueCredentialResponse)

    Properties:
        created_at: (createdAt) REQUIRED str

        owner: (owner) REQUIRED str

        recommended_max_age_days: (recommendedMaxAgeDays) REQUIRED int

        username: (username) REQUIRED str
    """

    # region fields

    created_at: str  # REQUIRED
    owner: str  # REQUIRED
    recommended_max_age_days: int  # REQUIRED
    username: str  # REQUIRED

    # endregion fields

    # region with_x methods

    def with_created_at(self, value: str) -> ApimodelCreateKeyValueCredentialResponse:
        self.created_at = value
        return self

    def with_owner(self, value: str) -> ApimodelCreateKeyValueCredentialResponse:
        self.owner = value
        return self

    def with_recommended_max_age_days(
        self, value: int
    ) -> ApimodelCreateKeyValueCredentialResponse:
        self.recommended_max_age_days = value
        return self

    def with_username(self, value: str) -> ApimodelCreateKeyValueCredentialResponse:
        self.username = value
        return self

    # endregion with_x methods

    # region to methods

    def to_dict(self, include_empty: bool = False) -> dict:
        result: dict = {}
        if hasattr(self, "created_at"):
            result["createdAt"] = str(self.created_at)
        elif include_empty:
            result["createdAt"] = ""
        if hasattr(self, "owner"):
            result["owner"] = str(self.owner)
        elif include_empty:
            result["owner"] = ""
        if hasattr(self, "recommended_max_age_days"):
            result["recommendedMaxAgeDays"] = int(self.recommended_max_age_days)
        elif include_empty:
            result["recommendedMaxAgeDays"] = 0
        if hasattr(self, "username"):
            result["username"] = str(self.username)
        elif include_empty:
            result["username"] = ""
        return result

    # endregion to methods

    # region static methods

    @classmethod
    def create(
        cls,
        created_at: str,
        owner: str,
        recommended_max_age_days: int,
        username: str,
        **kwargs,
    ) -> ApimodelCreateKeyValueCredentialResponse:
        instance = cls()
        instance.created_at = created_at
        instance.owner = owner
        instance.recommended_max_age_days = recommended_max_age_days
        instance.username = username
        return instance

    @classmethod
    def create_from_dict(
        cls, dict_: dict, include_empty: bool = False
    ) -> ApimodelCreateKeyValueCredentialResponse:
        instance = cls()
        if not dict_:
            return instance
        if "createdAt" in dict_ and dict_["createdAt"] is not None:
            instance.created_at = str(dict_["createdAt"])
        elif include_empty:
            instance.created_at = ""
        if "owner" in dict_ and dict_["owner"] is not None:
            instance.owner = str(dict_["owner"])
        elif include_empty:
            instance.owner = ""
        if (
            "recommendedMaxAgeDays" in dict_
            and dict_["recommendedMaxAgeDays"] is not None
        ):
            instance.recommended_max_age_days = int(dict_["recommendedMaxAgeDays"])
        elif include_empty:
            instance.recommended_max_age_days = 0
        if "username" in dict_ and dict_["username"] is not None:
            instance.username = str(dict_["username"])
        elif include_empty:
            instance.username = ""
        return instance

    @classmethod
    def create_many_from_dict(
        cls, dict_: dict, include_empty: bool = False
    ) -> Dict[str, ApimodelCreateKeyValueCredentialResponse]:
        return (
            {k: cls.create_from_dict(v, include_empty=include_empty) for k, v in dict_}
            if dict_
            else {}
        )

    @classmethod
    def create_many_from_list(
        cls, list_: list, include_empty: bool = False
    ) -> List[ApimodelCreateKeyValueCredentialResponse]:
        return (
            [cls.create_from_dict(i, include_empty=include_empty) for i in list_]
            if list_
            else []
        )

    @classmethod
    def create_from_any(
        cls, any_: any, include_empty: bool = False, many: bool = False
    ) -> Union[
        ApimodelCreateKeyValueCredentialResponse,
        List[ApimodelCreateKeyValueCredentialResponse],
        Dict[Any, ApimodelCreateKeyValueCredentialResponse],
    ]:
        if many:
            if isinstance(any_, dict):
                return cls.create_many_from_dict(any_, include_empty=include_empty)
            elif isinstance(any_, list):
                return cls.create_many_from_list(any_, include_empty=include_empty)
            else:
                raise ValueError()
        else:
            return cls.create_from_dict(any_, include_empty=include_empty)

    @staticmethod
    def get_field_info() -> Dict[str, str]:
        return {
            "createdAt": "created_at",
            "owner": "owner",
            "recommendedMaxAgeDays": "recommended_max_age_days",
            "username": "username",
        }

    @staticmethod
    def get_required_map() -> Dict[str, bool]:
        return {
            "createdAt": True,
            "owner": True,
            "recommendedMaxAgeDays": True,
            "username": True,
        }

    # endregion static methods



# Copyright (c) 2021 AccelByte Inc. All Rights Reserved.
# This is licensed software from AccelByte Inc, for limitations
# and restrictions contact your company contract manager.
#
# Code generated. DO NOT EDIT!

# template file: ags_py_codegen

# pylint: disable=duplicate-code
# pylint: disable=line-too-long
# pylint: disable=missing-function-docstring
# pylint: disable=missing-function-docstring
# pylint: disable=missing-module-docstring
# pylint: disable=too-many-arguments
# pylint: disable=too-many-branches
# pylint: disable=too-many-instance-attributes
# pylint: disable=too-many-lines
# pylint: disable=too-many-locals
# pylint: disable=too-many-public-methods
# pylint: disable=too-many-return-statements
# pylint: disable=too-many-statements
# pylint: disable=unused-import

from typing import Any, Dict, List, Optional, Tuple, Union

from ....core import HeaderStr
from ....core import get_namespace as get_services_namespace
from ....core import run_request
from ....core import run_request_async
from ....core import deprecated
from ....core import same_doc_as

from ..models import ModelsPartyData
from ..models import ModelsPartyPUTCustomAttributesRequest
from ..models import ModelsPartyPUTLimitSizeRequest
from ..models import RestapiErrorResponseBody

from ..operations.party import AdminGetPartyDataV1
from ..operations.party import AdminGetUserPartyV1
from ..operations.party import PublicGetPartyDataV1
from ..operations.party import PublicSetPartyLimitV1
from ..operations.party import PublicUpdatePartyAttributesV1


@deprecated
@same_doc_as(AdminGetPartyDataV1)
def admin_get_party_data_v1(
    party_id: str,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """admin get party data (adminGetPartyDataV1)

    Get party data in a namespace.

    Properties:
        url: /lobby/v1/admin/party/namespaces/{namespace}/parties/{partyId}

        method: GET

        tags: ["party"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        namespace: (namespace) REQUIRED str in path

        party_id: (partyId) REQUIRED str in path

    Responses:
        200: OK - ModelsPartyData (OK)

        400: Bad Request - RestapiErrorResponseBody (Bad Request)

        401: Unauthorized - RestapiErrorResponseBody (Unauthorized)

        403: Forbidden - RestapiErrorResponseBody (Forbidden)

        404: Not Found - RestapiErrorResponseBody (Not Found)

        500: Internal Server Error - RestapiErrorResponseBody (Internal Server Error)
    """
    if namespace is None:
        namespace, error = get_services_namespace(sdk=kwargs.get("sdk"))
        if error:
            return None, error
    request = AdminGetPartyDataV1.create(
        party_id=party_id,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@deprecated
@same_doc_as(AdminGetPartyDataV1)
async def admin_get_party_data_v1_async(
    party_id: str,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """admin get party data (adminGetPartyDataV1)

    Get party data in a namespace.

    Properties:
        url: /lobby/v1/admin/party/namespaces/{namespace}/parties/{partyId}

        method: GET

        tags: ["party"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        namespace: (namespace) REQUIRED str in path

        party_id: (partyId) REQUIRED str in path

    Responses:
        200: OK - ModelsPartyData (OK)

        400: Bad Request - RestapiErrorResponseBody (Bad Request)

        401: Unauthorized - RestapiErrorResponseBody (Unauthorized)

        403: Forbidden - RestapiErrorResponseBody (Forbidden)

        404: Not Found - RestapiErrorResponseBody (Not Found)

        500: Internal Server Error - RestapiErrorResponseBody (Internal Server Error)
    """
    if namespace is None:
        namespace, error = get_services_namespace(sdk=kwargs.get("sdk"))
        if error:
            return None, error
    request = AdminGetPartyDataV1.create(
        party_id=party_id,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@deprecated
@same_doc_as(AdminGetUserPartyV1)
def admin_get_user_party_v1(
    user_id: str,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """admin get user party data (adminGetUserPartyV1)

    Get party data in a namespace.

    Properties:
        url: /lobby/v1/admin/party/namespaces/{namespace}/users/{userId}/party

        method: GET

        tags: ["party"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        namespace: (namespace) REQUIRED str in path

        user_id: (userId) REQUIRED str in path

    Responses:
        200: OK - ModelsPartyData (OK)

        400: Bad Request - RestapiErrorResponseBody (Bad Request)

        401: Unauthorized - RestapiErrorResponseBody (Unauthorized)

        403: Forbidden - RestapiErrorResponseBody (Forbidden)

        404: Not Found - RestapiErrorResponseBody (Not Found)

        500: Internal Server Error - RestapiErrorResponseBody (Internal Server Error)
    """
    if namespace is None:
        namespace, error = get_services_namespace(sdk=kwargs.get("sdk"))
        if error:
            return None, error
    request = AdminGetUserPartyV1.create(
        user_id=user_id,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@deprecated
@same_doc_as(AdminGetUserPartyV1)
async def admin_get_user_party_v1_async(
    user_id: str,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """admin get user party data (adminGetUserPartyV1)

    Get party data in a namespace.

    Properties:
        url: /lobby/v1/admin/party/namespaces/{namespace}/users/{userId}/party

        method: GET

        tags: ["party"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        namespace: (namespace) REQUIRED str in path

        user_id: (userId) REQUIRED str in path

    Responses:
        200: OK - ModelsPartyData (OK)

        400: Bad Request - RestapiErrorResponseBody (Bad Request)

        401: Unauthorized - RestapiErrorResponseBody (Unauthorized)

        403: Forbidden - RestapiErrorResponseBody (Forbidden)

        404: Not Found - RestapiErrorResponseBody (Not Found)

        500: Internal Server Error - RestapiErrorResponseBody (Internal Server Error)
    """
    if namespace is None:
        namespace, error = get_services_namespace(sdk=kwargs.get("sdk"))
        if error:
            return None, error
    request = AdminGetUserPartyV1.create(
        user_id=user_id,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@deprecated
@same_doc_as(PublicGetPartyDataV1)
def public_get_party_data_v1(
    party_id: str,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """get party data by party id (publicGetPartyDataV1)

    Load party data in a namespace by Party ID.

    Properties:
        url: /lobby/v1/public/party/namespaces/{namespace}/parties/{partyId}

        method: GET

        tags: ["party"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        namespace: (namespace) REQUIRED str in path

        party_id: (partyId) REQUIRED str in path

    Responses:
        200: OK - ModelsPartyData (OK)

        400: Bad Request - RestapiErrorResponseBody (Bad Request)

        401: Unauthorized - RestapiErrorResponseBody (Unauthorized)

        403: Forbidden - RestapiErrorResponseBody (Forbidden)

        404: Not Found - RestapiErrorResponseBody (Not Found)

        500: Internal Server Error - RestapiErrorResponseBody (Internal Server Error)
    """
    if namespace is None:
        namespace, error = get_services_namespace(sdk=kwargs.get("sdk"))
        if error:
            return None, error
    request = PublicGetPartyDataV1.create(
        party_id=party_id,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@deprecated
@same_doc_as(PublicGetPartyDataV1)
async def public_get_party_data_v1_async(
    party_id: str,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """get party data by party id (publicGetPartyDataV1)

    Load party data in a namespace by Party ID.

    Properties:
        url: /lobby/v1/public/party/namespaces/{namespace}/parties/{partyId}

        method: GET

        tags: ["party"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        namespace: (namespace) REQUIRED str in path

        party_id: (partyId) REQUIRED str in path

    Responses:
        200: OK - ModelsPartyData (OK)

        400: Bad Request - RestapiErrorResponseBody (Bad Request)

        401: Unauthorized - RestapiErrorResponseBody (Unauthorized)

        403: Forbidden - RestapiErrorResponseBody (Forbidden)

        404: Not Found - RestapiErrorResponseBody (Not Found)

        500: Internal Server Error - RestapiErrorResponseBody (Internal Server Error)
    """
    if namespace is None:
        namespace, error = get_services_namespace(sdk=kwargs.get("sdk"))
        if error:
            return None, error
    request = PublicGetPartyDataV1.create(
        party_id=party_id,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@deprecated
@same_doc_as(PublicSetPartyLimitV1)
def public_set_party_limit_v1(
    body: ModelsPartyPUTLimitSizeRequest,
    party_id: str,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """set party limit (publicSetPartyLimitV1)

    Required valid user authorization


    Set party limit, only party leader can call this endpoint.

    Properties:
        url: /lobby/v1/public/party/namespaces/{namespace}/parties/{partyId}/limit

        method: PUT

        tags: ["party"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        body: (body) REQUIRED ModelsPartyPUTLimitSizeRequest in body

        namespace: (namespace) REQUIRED str in path

        party_id: (partyId) REQUIRED str in path

    Responses:
        200: OK - (OK)

        400: Bad Request - RestapiErrorResponseBody (Bad Request)

        401: Unauthorized - RestapiErrorResponseBody (Unauthorized)

        403: Forbidden - RestapiErrorResponseBody (Forbidden)

        404: Not Found - RestapiErrorResponseBody (Not Found)

        500: Internal Server Error - RestapiErrorResponseBody (Internal Server Error)
    """
    if namespace is None:
        namespace, error = get_services_namespace(sdk=kwargs.get("sdk"))
        if error:
            return None, error
    request = PublicSetPartyLimitV1.create(
        body=body,
        party_id=party_id,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@deprecated
@same_doc_as(PublicSetPartyLimitV1)
async def public_set_party_limit_v1_async(
    body: ModelsPartyPUTLimitSizeRequest,
    party_id: str,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """set party limit (publicSetPartyLimitV1)

    Required valid user authorization


    Set party limit, only party leader can call this endpoint.

    Properties:
        url: /lobby/v1/public/party/namespaces/{namespace}/parties/{partyId}/limit

        method: PUT

        tags: ["party"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        body: (body) REQUIRED ModelsPartyPUTLimitSizeRequest in body

        namespace: (namespace) REQUIRED str in path

        party_id: (partyId) REQUIRED str in path

    Responses:
        200: OK - (OK)

        400: Bad Request - RestapiErrorResponseBody (Bad Request)

        401: Unauthorized - RestapiErrorResponseBody (Unauthorized)

        403: Forbidden - RestapiErrorResponseBody (Forbidden)

        404: Not Found - RestapiErrorResponseBody (Not Found)

        500: Internal Server Error - RestapiErrorResponseBody (Internal Server Error)
    """
    if namespace is None:
        namespace, error = get_services_namespace(sdk=kwargs.get("sdk"))
        if error:
            return None, error
    request = PublicSetPartyLimitV1.create(
        body=body,
        party_id=party_id,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@deprecated
@same_doc_as(PublicUpdatePartyAttributesV1)
def public_update_party_attributes_v1(
    body: ModelsPartyPUTCustomAttributesRequest,
    party_id: str,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """update party attributes (publicUpdatePartyAttributesV1)

    Required valid user authorization


    update party attributes in a namespace.

    Properties:
        url: /lobby/v1/public/party/namespaces/{namespace}/parties/{partyId}/attributes

        method: PUT

        tags: ["party"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        body: (body) REQUIRED ModelsPartyPUTCustomAttributesRequest in body

        namespace: (namespace) REQUIRED str in path

        party_id: (partyId) REQUIRED str in path

    Responses:
        200: OK - ModelsPartyData (OK)

        400: Bad Request - RestapiErrorResponseBody (Bad Request)

        401: Unauthorized - RestapiErrorResponseBody (Unauthorized)

        403: Forbidden - RestapiErrorResponseBody (Forbidden)

        404: Not Found - RestapiErrorResponseBody (Not Found)

        412: Precondition Failed - RestapiErrorResponseBody (Precondition Failed)

        500: Internal Server Error - RestapiErrorResponseBody (Internal Server Error)
    """
    if namespace is None:
        namespace, error = get_services_namespace(sdk=kwargs.get("sdk"))
        if error:
            return None, error
    request = PublicUpdatePartyAttributesV1.create(
        body=body,
        party_id=party_id,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@deprecated
@same_doc_as(PublicUpdatePartyAttributesV1)
async def public_update_party_attributes_v1_async(
    body: ModelsPartyPUTCustomAttributesRequest,
    party_id: str,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """update party attributes (publicUpdatePartyAttributesV1)

    Required valid user authorization


    update party attributes in a namespace.

    Properties:
        url: /lobby/v1/public/party/namespaces/{namespace}/parties/{partyId}/attributes

        method: PUT

        tags: ["party"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        body: (body) REQUIRED ModelsPartyPUTCustomAttributesRequest in body

        namespace: (namespace) REQUIRED str in path

        party_id: (partyId) REQUIRED str in path

    Responses:
        200: OK - ModelsPartyData (OK)

        400: Bad Request - RestapiErrorResponseBody (Bad Request)

        401: Unauthorized - RestapiErrorResponseBody (Unauthorized)

        403: Forbidden - RestapiErrorResponseBody (Forbidden)

        404: Not Found - RestapiErrorResponseBody (Not Found)

        412: Precondition Failed - RestapiErrorResponseBody (Precondition Failed)

        500: Internal Server Error - RestapiErrorResponseBody (Internal Server Error)
    """
    if namespace is None:
        namespace, error = get_services_namespace(sdk=kwargs.get("sdk"))
        if error:
            return None, error
    request = PublicUpdatePartyAttributesV1.create(
        body=body,
        party_id=party_id,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )



# Copyright (c) 2021 AccelByte Inc. All Rights Reserved.
# This is licensed software from AccelByte Inc, for limitations
# and restrictions contact your company contract manager.
#
# Code generated. DO NOT EDIT!

# template file: ags_py_codegen

# AccelByte Gaming Services Iam Service

# pylint: disable=duplicate-code
# pylint: disable=line-too-long
# pylint: disable=missing-function-docstring
# pylint: disable=missing-module-docstring
# pylint: disable=too-many-arguments
# pylint: disable=too-many-branches
# pylint: disable=too-many-instance-attributes
# pylint: disable=too-many-lines
# pylint: disable=too-many-locals
# pylint: disable=too-many-public-methods
# pylint: disable=too-many-return-statements
# pylint: disable=too-many-statements
# pylint: disable=unused-import

from __future__ import annotations
from typing import Any, Dict, List, Optional, Tuple, Union

from ....core import Model


class OauthcommonASMetadataResponse(Model):
    """Oauthcommon AS metadata response (oauthcommon.ASMetadataResponse)

    Properties:
        authorization_endpoint: (authorization_endpoint) REQUIRED str

        code_challenge_methods_supported: (code_challenge_methods_supported) REQUIRED List[str]

        grant_types_supported: (grant_types_supported) REQUIRED List[str]

        issuer: (issuer) REQUIRED str

        jwks_uri: (jwks_uri) REQUIRED str

        registration_endpoint: (registration_endpoint) REQUIRED str

        response_types_supported: (response_types_supported) REQUIRED List[str]

        scopes_supported: (scopes_supported) REQUIRED List[str]

        token_endpoint: (token_endpoint) REQUIRED str

        token_endpoint_auth_methods_supported: (token_endpoint_auth_methods_supported) REQUIRED List[str]
    """

    # region fields

    authorization_endpoint: str  # REQUIRED
    code_challenge_methods_supported: List[str]  # REQUIRED
    grant_types_supported: List[str]  # REQUIRED
    issuer: str  # REQUIRED
    jwks_uri: str  # REQUIRED
    registration_endpoint: str  # REQUIRED
    response_types_supported: List[str]  # REQUIRED
    scopes_supported: List[str]  # REQUIRED
    token_endpoint: str  # REQUIRED
    token_endpoint_auth_methods_supported: List[str]  # REQUIRED

    # endregion fields

    # region with_x methods

    def with_authorization_endpoint(self, value: str) -> OauthcommonASMetadataResponse:
        self.authorization_endpoint = value
        return self

    def with_code_challenge_methods_supported(
        self, value: List[str]
    ) -> OauthcommonASMetadataResponse:
        self.code_challenge_methods_supported = value
        return self

    def with_grant_types_supported(
        self, value: List[str]
    ) -> OauthcommonASMetadataResponse:
        self.grant_types_supported = value
        return self

    def with_issuer(self, value: str) -> OauthcommonASMetadataResponse:
        self.issuer = value
        return self

    def with_jwks_uri(self, value: str) -> OauthcommonASMetadataResponse:
        self.jwks_uri = value
        return self

    def with_registration_endpoint(self, value: str) -> OauthcommonASMetadataResponse:
        self.registration_endpoint = value
        return self

    def with_response_types_supported(
        self, value: List[str]
    ) -> OauthcommonASMetadataResponse:
        self.response_types_supported = value
        return self

    def with_scopes_supported(self, value: List[str]) -> OauthcommonASMetadataResponse:
        self.scopes_supported = value
        return self

    def with_token_endpoint(self, value: str) -> OauthcommonASMetadataResponse:
        self.token_endpoint = value
        return self

    def with_token_endpoint_auth_methods_supported(
        self, value: List[str]
    ) -> OauthcommonASMetadataResponse:
        self.token_endpoint_auth_methods_supported = value
        return self

    # endregion with_x methods

    # region to methods

    def to_dict(self, include_empty: bool = False) -> dict:
        result: dict = {}
        if hasattr(self, "authorization_endpoint"):
            result["authorization_endpoint"] = str(self.authorization_endpoint)
        elif include_empty:
            result["authorization_endpoint"] = ""
        if hasattr(self, "code_challenge_methods_supported"):
            result["code_challenge_methods_supported"] = [
                str(i0) for i0 in self.code_challenge_methods_supported
            ]
        elif include_empty:
            result["code_challenge_methods_supported"] = []
        if hasattr(self, "grant_types_supported"):
            result["grant_types_supported"] = [
                str(i0) for i0 in self.grant_types_supported
            ]
        elif include_empty:
            result["grant_types_supported"] = []
        if hasattr(self, "issuer"):
            result["issuer"] = str(self.issuer)
        elif include_empty:
            result["issuer"] = ""
        if hasattr(self, "jwks_uri"):
            result["jwks_uri"] = str(self.jwks_uri)
        elif include_empty:
            result["jwks_uri"] = ""
        if hasattr(self, "registration_endpoint"):
            result["registration_endpoint"] = str(self.registration_endpoint)
        elif include_empty:
            result["registration_endpoint"] = ""
        if hasattr(self, "response_types_supported"):
            result["response_types_supported"] = [
                str(i0) for i0 in self.response_types_supported
            ]
        elif include_empty:
            result["response_types_supported"] = []
        if hasattr(self, "scopes_supported"):
            result["scopes_supported"] = [str(i0) for i0 in self.scopes_supported]
        elif include_empty:
            result["scopes_supported"] = []
        if hasattr(self, "token_endpoint"):
            result["token_endpoint"] = str(self.token_endpoint)
        elif include_empty:
            result["token_endpoint"] = ""
        if hasattr(self, "token_endpoint_auth_methods_supported"):
            result["token_endpoint_auth_methods_supported"] = [
                str(i0) for i0 in self.token_endpoint_auth_methods_supported
            ]
        elif include_empty:
            result["token_endpoint_auth_methods_supported"] = []
        return result

    # endregion to methods

    # region static methods

    @classmethod
    def create(
        cls,
        authorization_endpoint: str,
        code_challenge_methods_supported: List[str],
        grant_types_supported: List[str],
        issuer: str,
        jwks_uri: str,
        registration_endpoint: str,
        response_types_supported: List[str],
        scopes_supported: List[str],
        token_endpoint: str,
        token_endpoint_auth_methods_supported: List[str],
        **kwargs,
    ) -> OauthcommonASMetadataResponse:
        instance = cls()
        instance.authorization_endpoint = authorization_endpoint
        instance.code_challenge_methods_supported = code_challenge_methods_supported
        instance.grant_types_supported = grant_types_supported
        instance.issuer = issuer
        instance.jwks_uri = jwks_uri
        instance.registration_endpoint = registration_endpoint
        instance.response_types_supported = response_types_supported
        instance.scopes_supported = scopes_supported
        instance.token_endpoint = token_endpoint
        instance.token_endpoint_auth_methods_supported = (
            token_endpoint_auth_methods_supported
        )
        return instance

    @classmethod
    def create_from_dict(
        cls, dict_: dict, include_empty: bool = False
    ) -> OauthcommonASMetadataResponse:
        instance = cls()
        if not dict_:
            return instance
        if (
            "authorization_endpoint" in dict_
            and dict_["authorization_endpoint"] is not None
        ):
            instance.authorization_endpoint = str(dict_["authorization_endpoint"])
        elif include_empty:
            instance.authorization_endpoint = ""
        if (
            "code_challenge_methods_supported" in dict_
            and dict_["code_challenge_methods_supported"] is not None
        ):
            instance.code_challenge_methods_supported = [
                str(i0) for i0 in dict_["code_challenge_methods_supported"]
            ]
        elif include_empty:
            instance.code_challenge_methods_supported = []
        if (
            "grant_types_supported" in dict_
            and dict_["grant_types_supported"] is not None
        ):
            instance.grant_types_supported = [
                str(i0) for i0 in dict_["grant_types_supported"]
            ]
        elif include_empty:
            instance.grant_types_supported = []
        if "issuer" in dict_ and dict_["issuer"] is not None:
            instance.issuer = str(dict_["issuer"])
        elif include_empty:
            instance.issuer = ""
        if "jwks_uri" in dict_ and dict_["jwks_uri"] is not None:
            instance.jwks_uri = str(dict_["jwks_uri"])
        elif include_empty:
            instance.jwks_uri = ""
        if (
            "registration_endpoint" in dict_
            and dict_["registration_endpoint"] is not None
        ):
            instance.registration_endpoint = str(dict_["registration_endpoint"])
        elif include_empty:
            instance.registration_endpoint = ""
        if (
            "response_types_supported" in dict_
            and dict_["response_types_supported"] is not None
        ):
            instance.response_types_supported = [
                str(i0) for i0 in dict_["response_types_supported"]
            ]
        elif include_empty:
            instance.response_types_supported = []
        if "scopes_supported" in dict_ and dict_["scopes_supported"] is not None:
            instance.scopes_supported = [str(i0) for i0 in dict_["scopes_supported"]]
        elif include_empty:
            instance.scopes_supported = []
        if "token_endpoint" in dict_ and dict_["token_endpoint"] is not None:
            instance.token_endpoint = str(dict_["token_endpoint"])
        elif include_empty:
            instance.token_endpoint = ""
        if (
            "token_endpoint_auth_methods_supported" in dict_
            and dict_["token_endpoint_auth_methods_supported"] is not None
        ):
            instance.token_endpoint_auth_methods_supported = [
                str(i0) for i0 in dict_["token_endpoint_auth_methods_supported"]
            ]
        elif include_empty:
            instance.token_endpoint_auth_methods_supported = []
        return instance

    @classmethod
    def create_many_from_dict(
        cls, dict_: dict, include_empty: bool = False
    ) -> Dict[str, OauthcommonASMetadataResponse]:
        return (
            {k: cls.create_from_dict(v, include_empty=include_empty) for k, v in dict_}
            if dict_
            else {}
        )

    @classmethod
    def create_many_from_list(
        cls, list_: list, include_empty: bool = False
    ) -> List[OauthcommonASMetadataResponse]:
        return (
            [cls.create_from_dict(i, include_empty=include_empty) for i in list_]
            if list_
            else []
        )

    @classmethod
    def create_from_any(
        cls, any_: any, include_empty: bool = False, many: bool = False
    ) -> Union[
        OauthcommonASMetadataResponse,
        List[OauthcommonASMetadataResponse],
        Dict[Any, OauthcommonASMetadataResponse],
    ]:
        if many:
            if isinstance(any_, dict):
                return cls.create_many_from_dict(any_, include_empty=include_empty)
            elif isinstance(any_, list):
                return cls.create_many_from_list(any_, include_empty=include_empty)
            else:
                raise ValueError()
        else:
            return cls.create_from_dict(any_, include_empty=include_empty)

    @staticmethod
    def get_field_info() -> Dict[str, str]:
        return {
            "authorization_endpoint": "authorization_endpoint",
            "code_challenge_methods_supported": "code_challenge_methods_supported",
            "grant_types_supported": "grant_types_supported",
            "issuer": "issuer",
            "jwks_uri": "jwks_uri",
            "registration_endpoint": "registration_endpoint",
            "response_types_supported": "response_types_supported",
            "scopes_supported": "scopes_supported",
            "token_endpoint": "token_endpoint",
            "token_endpoint_auth_methods_supported": "token_endpoint_auth_methods_supported",
        }

    @staticmethod
    def get_required_map() -> Dict[str, bool]:
        return {
            "authorization_endpoint": True,
            "code_challenge_methods_supported": True,
            "grant_types_supported": True,
            "issuer": True,
            "jwks_uri": True,
            "registration_endpoint": True,
            "response_types_supported": True,
            "scopes_supported": True,
            "token_endpoint": True,
            "token_endpoint_auth_methods_supported": True,
        }

    # endregion static methods



# Copyright (c) 2021 AccelByte Inc. All Rights Reserved.
# This is licensed software from AccelByte Inc, for limitations
# and restrictions contact your company contract manager.
#
# Code generated. DO NOT EDIT!

# template file: ags_py_codegen

# AccelByte Gaming Services Statistics Service

# pylint: disable=duplicate-code
# pylint: disable=line-too-long
# pylint: disable=missing-function-docstring
# pylint: disable=missing-module-docstring
# pylint: disable=too-many-arguments
# pylint: disable=too-many-branches
# pylint: disable=too-many-instance-attributes
# pylint: disable=too-many-lines
# pylint: disable=too-many-locals
# pylint: disable=too-many-public-methods
# pylint: disable=too-many-return-statements
# pylint: disable=too-many-statements
# pylint: disable=unused-import

from __future__ import annotations
from typing import Any, Dict, List, Optional, Tuple, Union

from ....core import Model


class UserStatCycleItemInfo(Model):
    """User stat cycle item info (UserStatCycleItemInfo)

    Properties:
        created_at: (createdAt) REQUIRED str

        cycle_id: (cycleId) REQUIRED str

        cycle_name: (cycleName) REQUIRED str

        cycle_version: (cycleVersion) REQUIRED int

        namespace: (namespace) REQUIRED str

        stat_code: (statCode) REQUIRED str

        stat_name: (statName) REQUIRED str

        updated_at: (updatedAt) REQUIRED str

        user_id: (userId) REQUIRED str

        value: (value) REQUIRED float

        additional_data: (additionalData) OPTIONAL Dict[str, Any]

        tags: (tags) OPTIONAL List[str]
    """

    # region fields

    created_at: str  # REQUIRED
    cycle_id: str  # REQUIRED
    cycle_name: str  # REQUIRED
    cycle_version: int  # REQUIRED
    namespace: str  # REQUIRED
    stat_code: str  # REQUIRED
    stat_name: str  # REQUIRED
    updated_at: str  # REQUIRED
    user_id: str  # REQUIRED
    value: float  # REQUIRED
    additional_data: Dict[str, Any]  # OPTIONAL
    tags: List[str]  # OPTIONAL

    # endregion fields

    # region with_x methods

    def with_created_at(self, value: str) -> UserStatCycleItemInfo:
        self.created_at = value
        return self

    def with_cycle_id(self, value: str) -> UserStatCycleItemInfo:
        self.cycle_id = value
        return self

    def with_cycle_name(self, value: str) -> UserStatCycleItemInfo:
        self.cycle_name = value
        return self

    def with_cycle_version(self, value: int) -> UserStatCycleItemInfo:
        self.cycle_version = value
        return self

    def with_namespace(self, value: str) -> UserStatCycleItemInfo:
        self.namespace = value
        return self

    def with_stat_code(self, value: str) -> UserStatCycleItemInfo:
        self.stat_code = value
        return self

    def with_stat_name(self, value: str) -> UserStatCycleItemInfo:
        self.stat_name = value
        return self

    def with_updated_at(self, value: str) -> UserStatCycleItemInfo:
        self.updated_at = value
        return self

    def with_user_id(self, value: str) -> UserStatCycleItemInfo:
        self.user_id = value
        return self

    def with_value(self, value: float) -> UserStatCycleItemInfo:
        self.value = value
        return self

    def with_additional_data(self, value: Dict[str, Any]) -> UserStatCycleItemInfo:
        self.additional_data = value
        return self

    def with_tags(self, value: List[str]) -> UserStatCycleItemInfo:
        self.tags = value
        return self

    # endregion with_x methods

    # region to methods

    def to_dict(self, include_empty: bool = False) -> dict:
        result: dict = {}
        if hasattr(self, "created_at"):
            result["createdAt"] = str(self.created_at)
        elif include_empty:
            result["createdAt"] = ""
        if hasattr(self, "cycle_id"):
            result["cycleId"] = str(self.cycle_id)
        elif include_empty:
            result["cycleId"] = ""
        if hasattr(self, "cycle_name"):
            result["cycleName"] = str(self.cycle_name)
        elif include_empty:
            result["cycleName"] = ""
        if hasattr(self, "cycle_version"):
            result["cycleVersion"] = int(self.cycle_version)
        elif include_empty:
            result["cycleVersion"] = 0
        if hasattr(self, "namespace"):
            result["namespace"] = str(self.namespace)
        elif include_empty:
            result["namespace"] = ""
        if hasattr(self, "stat_code"):
            result["statCode"] = str(self.stat_code)
        elif include_empty:
            result["statCode"] = ""
        if hasattr(self, "stat_name"):
            result["statName"] = str(self.stat_name)
        elif include_empty:
            result["statName"] = ""
        if hasattr(self, "updated_at"):
            result["updatedAt"] = str(self.updated_at)
        elif include_empty:
            result["updatedAt"] = ""
        if hasattr(self, "user_id"):
            result["userId"] = str(self.user_id)
        elif include_empty:
            result["userId"] = ""
        if hasattr(self, "value"):
            result["value"] = float(self.value)
        elif include_empty:
            result["value"] = 0.0
        if hasattr(self, "additional_data"):
            result["additionalData"] = {
                str(k0): v0 for k0, v0 in self.additional_data.items()
            }
        elif include_empty:
            result["additionalData"] = {}
        if hasattr(self, "tags"):
            result["tags"] = [str(i0) for i0 in self.tags]
        elif include_empty:
            result["tags"] = []
        return result

    # endregion to methods

    # region static methods

    @classmethod
    def create(
        cls,
        created_at: str,
        cycle_id: str,
        cycle_name: str,
        cycle_version: int,
        namespace: str,
        stat_code: str,
        stat_name: str,
        updated_at: str,
        user_id: str,
        value: float,
        additional_data: Optional[Dict[str, Any]] = None,
        tags: Optional[List[str]] = None,
        **kwargs,
    ) -> UserStatCycleItemInfo:
        instance = cls()
        instance.created_at = created_at
        instance.cycle_id = cycle_id
        instance.cycle_name = cycle_name
        instance.cycle_version = cycle_version
        instance.namespace = namespace
        instance.stat_code = stat_code
        instance.stat_name = stat_name
        instance.updated_at = updated_at
        instance.user_id = user_id
        instance.value = value
        if additional_data is not None:
            instance.additional_data = additional_data
        if tags is not None:
            instance.tags = tags
        return instance

    @classmethod
    def create_from_dict(
        cls, dict_: dict, include_empty: bool = False
    ) -> UserStatCycleItemInfo:
        instance = cls()
        if not dict_:
            return instance
        if "createdAt" in dict_ and dict_["createdAt"] is not None:
            instance.created_at = str(dict_["createdAt"])
        elif include_empty:
            instance.created_at = ""
        if "cycleId" in dict_ and dict_["cycleId"] is not None:
            instance.cycle_id = str(dict_["cycleId"])
        elif include_empty:
            instance.cycle_id = ""
        if "cycleName" in dict_ and dict_["cycleName"] is not None:
            instance.cycle_name = str(dict_["cycleName"])
        elif include_empty:
            instance.cycle_name = ""
        if "cycleVersion" in dict_ and dict_["cycleVersion"] is not None:
            instance.cycle_version = int(dict_["cycleVersion"])
        elif include_empty:
            instance.cycle_version = 0
        if "namespace" in dict_ and dict_["namespace"] is not None:
            instance.namespace = str(dict_["namespace"])
        elif include_empty:
            instance.namespace = ""
        if "statCode" in dict_ and dict_["statCode"] is not None:
            instance.stat_code = str(dict_["statCode"])
        elif include_empty:
            instance.stat_code = ""
        if "statName" in dict_ and dict_["statName"] is not None:
            instance.stat_name = str(dict_["statName"])
        elif include_empty:
            instance.stat_name = ""
        if "updatedAt" in dict_ and dict_["updatedAt"] is not None:
            instance.updated_at = str(dict_["updatedAt"])
        elif include_empty:
            instance.updated_at = ""
        if "userId" in dict_ and dict_["userId"] is not None:
            instance.user_id = str(dict_["userId"])
        elif include_empty:
            instance.user_id = ""
        if "value" in dict_ and dict_["value"] is not None:
            instance.value = float(dict_["value"])
        elif include_empty:
            instance.value = 0.0
        if "additionalData" in dict_ and dict_["additionalData"] is not None:
            instance.additional_data = {
                str(k0): v0 for k0, v0 in dict_["additionalData"].items()
            }
        elif include_empty:
            instance.additional_data = {}
        if "tags" in dict_ and dict_["tags"] is not None:
            instance.tags = [str(i0) for i0 in dict_["tags"]]
        elif include_empty:
            instance.tags = []
        return instance

    @classmethod
    def create_many_from_dict(
        cls, dict_: dict, include_empty: bool = False
    ) -> Dict[str, UserStatCycleItemInfo]:
        return (
            {k: cls.create_from_dict(v, include_empty=include_empty) for k, v in dict_}
            if dict_
            else {}
        )

    @classmethod
    def create_many_from_list(
        cls, list_: list, include_empty: bool = False
    ) -> List[UserStatCycleItemInfo]:
        return (
            [cls.create_from_dict(i, include_empty=include_empty) for i in list_]
            if list_
            else []
        )

    @classmethod
    def create_from_any(
        cls, any_: any, include_empty: bool = False, many: bool = False
    ) -> Union[
        UserStatCycleItemInfo,
        List[UserStatCycleItemInfo],
        Dict[Any, UserStatCycleItemInfo],
    ]:
        if many:
            if isinstance(any_, dict):
                return cls.create_many_from_dict(any_, include_empty=include_empty)
            elif isinstance(any_, list):
                return cls.create_many_from_list(any_, include_empty=include_empty)
            else:
                raise ValueError()
        else:
            return cls.create_from_dict(any_, include_empty=include_empty)

    @staticmethod
    def get_field_info() -> Dict[str, str]:
        return {
            "createdAt": "created_at",
            "cycleId": "cycle_id",
            "cycleName": "cycle_name",
            "cycleVersion": "cycle_version",
            "namespace": "namespace",
            "statCode": "stat_code",
            "statName": "stat_name",
            "updatedAt": "updated_at",
            "userId": "user_id",
            "value": "value",
            "additionalData": "additional_data",
            "tags": "tags",
        }

    @staticmethod
    def get_required_map() -> Dict[str, bool]:
        return {
            "createdAt": True,
            "cycleId": True,
            "cycleName": True,
            "cycleVersion": True,
            "namespace": True,
            "statCode": True,
            "statName": True,
            "updatedAt": True,
            "userId": True,
            "value": True,
            "additionalData": False,
            "tags": False,
        }

    # endregion static methods



# Copyright (c) 2021 AccelByte Inc. All Rights Reserved.
# This is licensed software from AccelByte Inc, for limitations
# and restrictions contact your company contract manager.
#
# Code generated. DO NOT EDIT!

# template file: ags_py_codegen

"""Auto-generated package that contains models used by the Custom Service Manager."""

__version__ = "1.34.0"
__author__ = "AccelByte"
__email__ = "dev@accelbyte.net"

# pylint: disable=line-too-long

# app
from .wrappers import create_app_v1
from .wrappers import create_app_v1_async
from .wrappers import delete_app_v1
from .wrappers import delete_app_v1_async
from .wrappers import get_app_list_v1
from .wrappers import get_app_list_v1_async
from .wrappers import get_app_release_v1
from .wrappers import get_app_release_v1_async
from .wrappers import get_app_v1
from .wrappers import get_app_v1_async
from .wrappers import start_app_v1
from .wrappers import start_app_v1_async
from .wrappers import stop_app_v1
from .wrappers import stop_app_v1_async
from .wrappers import update_app_v1
from .wrappers import update_app_v1_async

# app_status_progress_v4
from .wrappers import get_app_status_progress_v4
from .wrappers import get_app_status_progress_v4_async

# app_ui
from .wrappers import create_app_ui
from .wrappers import create_app_ui_async
from .wrappers import delete_app_ui
from .wrappers import delete_app_ui_async
from .wrappers import list_app_ui
from .wrappers import list_app_ui_async
from .wrappers import upload_app_ui_file
from .wrappers import upload_app_ui_file_async

# app_v2
from .wrappers import apply_app_config_v2
from .wrappers import apply_app_config_v2_async
from .wrappers import create_app_v2
from .wrappers import create_app_v2_async
from .wrappers import delete_app_v2
from .wrappers import delete_app_v2_async
from .wrappers import get_app_list_v2
from .wrappers import get_app_list_v2_async
from .wrappers import get_app_v2
from .wrappers import get_app_v2_async
from .wrappers import start_app_v2
from .wrappers import start_app_v2_async
from .wrappers import stop_app_v2
from .wrappers import stop_app_v2_async
from .wrappers import update_app_resources_resource_limit_form_v2
from .wrappers import update_app_resources_resource_limit_form_v2_async
from .wrappers import update_app_resources_v2
from .wrappers import update_app_resources_v2_async
from .wrappers import update_app_v2
from .wrappers import update_app_v2_async

# app_v4
from .wrappers import get_app_debug_info_v4
from .wrappers import get_app_debug_info_v4_async
from .wrappers import update_app_debug_mode_v4
from .wrappers import update_app_debug_mode_v4_async

# app_v5
from .wrappers import create_app_v5
from .wrappers import create_app_v5_async

# async_messaging
from .wrappers import create_subscription_handler
from .wrappers import create_subscription_handler_async
from .wrappers import create_topic_handler
from .wrappers import create_topic_handler_async
from .wrappers import delete_topic_handler
from .wrappers import delete_topic_handler_async
from .wrappers import list_topics_handler
from .wrappers import list_topics_handler_async
from .wrappers import unsubscribe_topic_handler
from .wrappers import unsubscribe_topic_handler_async

# configuration
from .wrappers import delete_secret_v1
from .wrappers import delete_secret_v1_async
from .wrappers import delete_variable_v1
from .wrappers import delete_variable_v1_async
from .wrappers import get_list_of_secrets_v1
from .wrappers import get_list_of_secrets_v1_async
from .wrappers import get_list_of_variables_v1
from .wrappers import get_list_of_variables_v1_async
from .wrappers import save_secret_v1
from .wrappers import save_secret_v1_async
from .wrappers import save_variable_v1
from .wrappers import save_variable_v1_async
from .wrappers import update_secret_v1
from .wrappers import update_secret_v1_async
from .wrappers import update_variable_v1
from .wrappers import update_variable_v1_async

# configuration_v2
from .wrappers import delete_secret_v2
from .wrappers import delete_secret_v2_async
from .wrappers import delete_variable_v2
from .wrappers import delete_variable_v2_async
from .wrappers import get_list_of_secrets_v2
from .wrappers import get_list_of_secrets_v2_async
from .wrappers import get_list_of_variables_v2
from .wrappers import get_list_of_variables_v2_async
from .wrappers import save_secret_v2
from .wrappers import save_secret_v2_async
from .wrappers import save_variable_v2
from .wrappers import save_variable_v2_async
from .wrappers import update_secret_v2
from .wrappers import update_secret_v2_async
from .wrappers import update_variable_v2
from .wrappers import update_variable_v2_async

# deployment
from .wrappers import create_deployment_v1
from .wrappers import create_deployment_v1_async
from .wrappers import delete_deployment_v1
from .wrappers import delete_deployment_v1_async
from .wrappers import get_deployment_v1
from .wrappers import get_deployment_v1_async
from .wrappers import get_list_of_deployment_v1
from .wrappers import get_list_of_deployment_v1_async

# deployment_v2
from .wrappers import create_deployment_v2
from .wrappers import create_deployment_v2_async
from .wrappers import delete_deployment_v2
from .wrappers import delete_deployment_v2_async
from .wrappers import get_deployment_v2
from .wrappers import get_deployment_v2_async
from .wrappers import get_list_of_deployment_v2
from .wrappers import get_list_of_deployment_v2_async

# extend_files
from .wrappers import get_extend_file
from .wrappers import get_extend_file_async

# image
from .wrappers import delete_app_images_v1
from .wrappers import delete_app_images_v1_async
from .wrappers import get_app_image_list_v1
from .wrappers import get_app_image_list_v1_async

# image_v2
from .wrappers import delete_app_images_v2
from .wrappers import delete_app_images_v2_async
from .wrappers import get_app_image_list_v2
from .wrappers import get_app_image_list_v2_async

# managed_resources
from .wrappers import create_new_no_sql_database_credential_v2
from .wrappers import create_new_no_sql_database_credential_v2_async
from .wrappers import create_no_sql_cluster_v2
from .wrappers import create_no_sql_cluster_v2_async
from .wrappers import create_no_sql_database_credential_v2
from .wrappers import create_no_sql_database_credential_v2_async
from .wrappers import create_no_sql_database_v2
from .wrappers import create_no_sql_database_v2_async
from .wrappers import delete_no_sql_cluster_v2
from .wrappers import delete_no_sql_cluster_v2_async
from .wrappers import delete_no_sql_database_v2
from .wrappers import delete_no_sql_database_v2_async
from .wrappers import get_no_sql_access_tunnel_v2
from .wrappers import get_no_sql_access_tunnel_v2_async
from .wrappers import get_no_sql_app_list_v2
from .wrappers import get_no_sql_app_list_v2_async
from .wrappers import get_no_sql_cluster_v2
from .wrappers import get_no_sql_cluster_v2_async
from .wrappers import get_no_sql_database_v2
from .wrappers import get_no_sql_database_v2_async
from .wrappers import start_no_sql_cluster_v2
from .wrappers import start_no_sql_cluster_v2_async
from .wrappers import stop_no_sql_cluster_v2
from .wrappers import stop_no_sql_cluster_v2_async
from .wrappers import update_no_sql_cluster_v2
from .wrappers import update_no_sql_cluster_v2_async

# managed_resources_key_value
from .wrappers import create_key_value_cluster_v2
from .wrappers import create_key_value_cluster_v2_async
from .wrappers import create_key_value_credential_v2
from .wrappers import create_key_value_credential_v2_async
from .wrappers import delete_key_value_cluster_v2
from .wrappers import delete_key_value_cluster_v2_async
from .wrappers import get_integration_app_key_value_cluster_v2
from .wrappers import get_integration_app_key_value_cluster_v2_async
from .wrappers import get_key_value_cluster_limit_config_v2
from .wrappers import get_key_value_cluster_limit_config_v2_async
from .wrappers import get_key_value_cluster_v2
from .wrappers import get_key_value_cluster_v2_async
from .wrappers import get_list_integrated_app_key_value_cluster_v2
from .wrappers import get_list_integrated_app_key_value_cluster_v2_async
from .wrappers import integrate_app_key_value_cluster_v2
from .wrappers import integrate_app_key_value_cluster_v2_async
from .wrappers import list_key_value_cluster_v2
from .wrappers import list_key_value_cluster_v2_async
from .wrappers import remove_integration_app_key_value_cluster_v2
from .wrappers import remove_integration_app_key_value_cluster_v2_async
from .wrappers import update_key_value_cluster_v2
from .wrappers import update_key_value_cluster_v2_async

# managed_resources_sql
from .wrappers import create_sql_cluster_v2
from .wrappers import create_sql_cluster_v2_async
from .wrappers import create_sql_database_credential_v2
from .wrappers import create_sql_database_credential_v2_async
from .wrappers import create_sql_database_v2
from .wrappers import create_sql_database_v2_async
from .wrappers import delete_sql_cluster_v2
from .wrappers import delete_sql_cluster_v2_async
from .wrappers import delete_sql_database_v2
from .wrappers import delete_sql_database_v2_async
from .wrappers import get_sql_app_list_v2
from .wrappers import get_sql_app_list_v2_async
from .wrappers import get_sql_cluster_v2
from .wrappers import get_sql_cluster_v2_async
from .wrappers import get_sql_database_v2
from .wrappers import get_sql_database_v2_async
from .wrappers import start_sql_cluster_v2
from .wrappers import start_sql_cluster_v2_async
from .wrappers import stop_sql_cluster_v2
from .wrappers import stop_sql_cluster_v2_async
from .wrappers import update_sql_cluster_v2
from .wrappers import update_sql_cluster_v2_async

# messages
from .wrappers import public_get_messages
from .wrappers import public_get_messages_async

# notification_subscription
from .wrappers import bulk_save_subscription_app_notification_v2
from .wrappers import bulk_save_subscription_app_notification_v2_async
from .wrappers import delete_subscription_app_notification_by_user_idv2
from .wrappers import delete_subscription_app_notification_by_user_idv2_async
from .wrappers import delete_subscription_app_notification_v2
from .wrappers import delete_subscription_app_notification_v2_async
from .wrappers import get_notification_subscriber_list_v2
from .wrappers import get_notification_subscriber_list_v2_async
from .wrappers import get_subscription_v2_handler
from .wrappers import get_subscription_v2_handler_async
from .wrappers import subscribe_app_notification_v2
from .wrappers import subscribe_app_notification_v2_async
from .wrappers import subscribe_v2_handler
from .wrappers import subscribe_v2_handler_async
from .wrappers import unsubscribe_v2_handler
from .wrappers import unsubscribe_v2_handler_async

# notification_subscription_v3
from .wrappers import delete_subscription_app_notification_v3
from .wrappers import delete_subscription_app_notification_v3_async
from .wrappers import get_notification_subscriber_list_v3
from .wrappers import get_notification_subscriber_list_v3_async

# resources_limits
from .wrappers import get_resources_limits
from .wrappers import get_resources_limits_async



# Copyright (c) 2025 AccelByte Inc. All Rights Reserved.
# This is licensed software from AccelByte Inc, for limitations
# and restrictions contact your company contract manager.

from typing import Dict, Optional

from ...core import (
    SDK,
    AccelByteSDK,
    create_basic_authentication,
)

from ...api import iam


def logout_internal(
    x_additional_headers: Optional[Dict[str, str]] = None,
    sdk: Optional[AccelByteSDK] = None,
    **kwargs,
):
    sdk = sdk or SDK

    access_token, error = sdk.get_access_token()
    if error:
        return None, error

    client_auth, error = sdk.get_client_auth()
    if error:
        return None, error

    x_additional_headers = x_additional_headers or {}
    x_additional_headers["Authorization"] = create_basic_authentication(*client_auth)

    _, error = iam.token_revocation_v3(
        token=access_token,
        x_additional_headers=x_additional_headers,
        sdk=sdk,
        **kwargs,
    )
    if error:
        return None, error

    _, error = sdk.remove_token()
    if error:
        return None, error

    return None, None


async def logout_async_internal(
    x_additional_headers: Optional[Dict[str, str]] = None,
    sdk: Optional[AccelByteSDK] = None,
    **kwargs,
):
    sdk = sdk or SDK

    access_token, error = sdk.get_access_token()
    if error:
        return None, error

    client_auth, error = sdk.get_client_auth()
    if error:
        return None, error

    x_additional_headers = x_additional_headers or {}
    x_additional_headers["Authorization"] = create_basic_authentication(*client_auth)

    _, error = await iam.token_revocation_v3_async(
        token=access_token,
        x_additional_headers=x_additional_headers,
        sdk=sdk,
        **kwargs,
    )
    if error:
        return None, error

    _, error = sdk.remove_token()
    if error:
        return None, error

    return None, None


__all__ = [
    "logout_internal",
    "logout_async_internal",
]



# Copyright (c) 2021 AccelByte Inc. All Rights Reserved.
# This is licensed software from AccelByte Inc, for limitations
# and restrictions contact your company contract manager.
#
# Code generated. DO NOT EDIT!

# template file: ags_py_codegen

"""Auto-generated package that contains models used by the AccelByte Gaming Services Platform Service."""

__version__ = "6.14.0"
__author__ = "AccelByte"
__email__ = "dev@accelbyte.net"

# pylint: disable=line-too-long

from .admin_get_iap_order_line_items import AdminGetIAPOrderLineItems
from .admin_get_steam_job_info import AdminGetSteamJobInfo
from .admin_refund_iap_order import AdminRefundIAPOrder
from .admin_reset_steam_job_time import AdminResetSteamJobTime
from .admin_sync_oculus_subsc_25b739 import AdminSyncOculusSubscriptions
from .admin_sync_steam_abnorm_be4791 import AdminSyncSteamAbnormalTransaction
from .admin_sync_steam_iap_by_48029c import AdminSyncSteamIAPByTransaction
from .admin_sync_twitch_drops_61485a import AdminSyncTwitchDropsEntitlement
from .delete_apple_iap_config import DeleteAppleIAPConfig
from .delete_epic_games_iap_config import DeleteEpicGamesIAPConfig
from .delete_google_iap_config import DeleteGoogleIAPConfig
from .delete_iap_item_config import DeleteIAPItemConfig
from .delete_oculus_iap_config import DeleteOculusIAPConfig
from .delete_playstation_iap_config import DeletePlaystationIAPConfig
from .delete_steam_iap_config import DeleteSteamIAPConfig
from .delete_twitch_iap_config import DeleteTwitchIAPConfig
from .delete_xbl_ap_config import DeleteXblAPConfig
from .get_apple_config_version import GetAppleConfigVersion
from .get_apple_iap_config import GetAppleIAPConfig
from .get_epic_games_iap_config import GetEpicGamesIAPConfig
from .get_google_iap_config import GetGoogleIAPConfig
from .get_iap_item_config import GetIAPItemConfig
from .get_iap_item_mapping import GetIAPItemMapping
from .get_iap_item_mapping import PlatformEnum as GetIAPItemMappingPlatformEnum
from .get_iap_order_consume_details import GetIAPOrderConsumeDetails
from .get_oculus_iap_config import GetOculusIAPConfig
from .get_play_station_iap_config import GetPlayStationIAPConfig
from .get_steam_iap_config import GetSteamIAPConfig
from .get_twitch_iap_config import GetTwitchIAPConfig
from .get_xbl_iap_config import GetXblIAPConfig
from .mock_fulfill_iap_item import MockFulfillIAPItem
from .public_fulfill_apple_iap_item import PublicFulfillAppleIAPItem
from .public_fulfill_google_iap_item import PublicFulfillGoogleIAPItem
from .public_reconcile_play_s_7ac4de import PublicReconcilePlayStationStore
from .public_reconcile_play_s_6be6c0 import (
    PublicReconcilePlayStationStoreWithMultipleServiceLabels,
)
from .query_abnormal_transactions import QueryAbnormalTransactions
from .query_all_user_iap_orders import QueryAllUserIAPOrders
from .query_steam_report_histories import QuerySteamReportHistories
from .query_steam_report_histories import (
    ProcessStatusEnum as QuerySteamReportHistoriesProcessStatusEnum,
)
from .query_user_iap_consume_history import QueryUserIAPConsumeHistory
from .query_user_iap_consume_history import (
    StatusEnum as QueryUserIAPConsumeHistoryStatusEnum,
    TypeEnum as QueryUserIAPConsumeHistoryTypeEnum,
)
from .query_user_iap_orders import QueryUserIAPOrders
from .query_user_iap_orders import (
    StatusEnum as QueryUserIAPOrdersStatusEnum,
    TypeEnum as QueryUserIAPOrdersTypeEnum,
)
from .sync_epic_games_inventory import SyncEpicGamesInventory
from .sync_oculus_consumable__f6c91d import SyncOculusConsumableEntitlements
from .sync_oculus_subscriptions import SyncOculusSubscriptions
from .sync_steam_abnormal_tra_4db4c2 import SyncSteamAbnormalTransaction
from .sync_steam_iap_by_transaction import SyncSteamIAPByTransaction
from .sync_steam_inventory import SyncSteamInventory
from .sync_twitch_drops_entitlement import SyncTwitchDropsEntitlement
from .sync_twitch_drops_entit_0c09ca import SyncTwitchDropsEntitlement1
from .sync_xbox_inventory import SyncXboxInventory
from .update_apple_iap_config import UpdateAppleIAPConfig
from .update_apple_p8_file import UpdateAppleP8File
from .update_epic_games_iap_config import UpdateEpicGamesIAPConfig
from .update_google_iap_config import UpdateGoogleIAPConfig
from .update_google_p12_file import UpdateGoogleP12File
from .update_iap_item_config import UpdateIAPItemConfig
from .update_oculus_iap_config import UpdateOculusIAPConfig
from .update_playstation_iap_config import UpdatePlaystationIAPConfig
from .update_steam_iap_config import UpdateSteamIAPConfig
from .update_twitch_iap_config import UpdateTwitchIAPConfig
from .update_xbl_bp_cert_file import UpdateXblBPCertFile
from .update_xbl_iap_config import UpdateXblIAPConfig
from .v2_public_fulfill_apple_ebaa63 import V2PublicFulfillAppleIAPItem
from .validate_existed_playst_881975 import ValidateExistedPlaystationIAPConfig
from .validate_playstation_ia_7dc74e import ValidatePlaystationIAPConfig



# Copyright (c) 2021 AccelByte Inc. All Rights Reserved.
# This is licensed software from AccelByte Inc, for limitations
# and restrictions contact your company contract manager.

from .v1 import login
from .v1 import login_async
from .v1 import login_client
from .v1 import login_client_async
from .v1 import login_platform
from .v1 import login_platform_async
from .v1 import login_user
from .v1 import login_user_async
from .v1 import refresh_login
from .v1 import refresh_login_async
from .v1 import logout
from .v1 import logout_async

# TODO: uncomment to replace with .v2, remember to remove .v1
# from .v2 import login
# from .v2 import login_async
# from .v2 import login_client
# from .v2 import login_client_async
# from .v2 import login_platform
# from .v2 import login_platform_async
# from .v2 import login_user
# from .v2 import login_user_async
# from .v2 import refresh_login
# from .v2 import refresh_login_async
# from .v2 import logout
# from .v2 import logout_async

# to be deprecated:

from .v1 import LoginTimerBase
from .v1 import LoginClientTimer
from .v1 import LoginPlatformTimer
from .v1 import LoginUserTimer
from .v1 import RefreshLoginTimer
from .v1 import OnDemandTokenRefresher
from .v1 import set_on_demand_token_refresher

# same:

from .v2 import DEFAULT_AUTO_REFRESH
from .v2 import DEFAULT_REFRESH_RATE
from .v2 import convert_bearer_auth_token_to_oauth_token_dict
from .v2 import parse_access_token



# Copyright (c) 2021 AccelByte Inc. All Rights Reserved.
# This is licensed software from AccelByte Inc, for limitations
# and restrictions contact your company contract manager.

from __future__ import annotations

import threading

from abc import ABC, abstractmethod
from typing import Dict, List, Optional, Tuple, Union

from ...core import (
    SDK,
    AccelByteSDK,
    HttpResponse,
    Operation,
    Timer,
    TimerStatus,
    TokenRepository,
    deprecated,
)

from ._login_const import (
    DEFAULT_REFRESH_RATE,
    DEFAULT_AUTO_REFRESH,
    DEFAULT_TIMER_INTERVAL,
    DEFAULT_TIMER_REFRESH_RATE,
    DEFAULT_TIMER_REPEAT_ON_EXCEPTION,
    AUTH_SERVICE_LOGGER,
    REFRESH_KWARGS,
)
from ._login_client import login_client_internal, login_client_async_internal
from ._login_platform import login_platform_internal, login_platform_async_internal
from ._login_user import login_user_internal, login_user_async_internal
from ._login_refresh import (
    login_refresh_internal,
    login_refresh_async_internal,
    try_refresh_login_internal,
    try_refresh_login_async_internal,
)
from ._logout import logout_internal, logout_async_internal


# region v1


@deprecated(
    message="2025-08-15 - This method is obsolete.",
    replacement="accelbyte_py_sdk.services.auth.v2.login_client",
)
def login_client(
    client_id: Optional[str] = None,
    client_secret: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs,
):
    token, error = login_client_internal(
        client_id=client_id,
        client_secret=client_secret,
        x_additional_headers=x_additional_headers,
        **kwargs,
    )
    if error:
        return None, error

    auto_refresh = kwargs.pop("auto_refresh", DEFAULT_AUTO_REFRESH)
    refresh_rate = kwargs.pop("refresh_rate", DEFAULT_REFRESH_RATE)
    if auto_refresh:
        set_on_demand_token_refresher(
            token_refresher=OnDemandTokenRefresher(
                refresher=LoginClientTimer(
                    0,
                    client_id=client_id,
                    client_secret=client_secret,
                    x_additional_headers=x_additional_headers,
                    **kwargs,
                ),
                refresh_rate=refresh_rate,
            ),
            **kwargs,
        )

    return token, error


@deprecated(
    message="2025-08-15 - This method is obsolete.",
    replacement="accelbyte_py_sdk.services.auth.v2.login_client_async",
)
async def login_client_async(
    client_id: Optional[str] = None,
    client_secret: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs,
):
    token, error = await login_client_async_internal(
        client_id=client_id,
        client_secret=client_secret,
        x_additional_headers=x_additional_headers,
        **kwargs,
    )
    if error:
        return None, error

    auto_refresh = kwargs.pop("auto_refresh", DEFAULT_AUTO_REFRESH)
    refresh_rate = kwargs.pop("refresh_rate", DEFAULT_REFRESH_RATE)
    if auto_refresh:
        set_on_demand_token_refresher(
            token_refresher=OnDemandTokenRefresher(
                refresher=LoginClientTimer(
                    0,
                    client_id=client_id,
                    client_secret=client_secret,
                    x_additional_headers=x_additional_headers,
                    **kwargs,
                ),
                refresh_rate=refresh_rate,
            ),
            **kwargs,
        )

    return token, error


@deprecated(
    message="2025-08-15 - This method is obsolete.",
    replacement="accelbyte_py_sdk.services.auth.v2.login_platform",
)
def login_platform(
    platform_id: str,
    platform_token: str,
    refresh_if_possible: bool = False,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs,
):
    token, error = login_platform_internal(
        platform_id=platform_id,
        platform_token=platform_token,
        refresh_if_possible=refresh_if_possible,
        x_additional_headers=x_additional_headers,
        **kwargs,
    )
    if error:
        return None, error

    auto_refresh = kwargs.pop("auto_refresh", DEFAULT_AUTO_REFRESH)
    refresh_rate = kwargs.pop("refresh_rate", DEFAULT_REFRESH_RATE)
    if auto_refresh:
        set_on_demand_token_refresher(
            token_refresher=OnDemandTokenRefresher(
                refresher=LoginPlatformTimer(
                    0,
                    platform_id=platform_id,
                    platform_token=platform_token,
                    x_additional_headers=x_additional_headers,
                    **kwargs,
                ),
                refresh_rate=refresh_rate,
            ),
            **kwargs,
        )

    return token, None


@deprecated(
    message="2025-08-15 - This method is obsolete.",
    replacement="accelbyte_py_sdk.services.auth.v2.login_platform_async",
)
async def login_platform_async(
    platform_id: str,
    platform_token: str,
    refresh_if_possible: bool = False,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs,
):
    token, error = await login_platform_async_internal(
        platform_id=platform_id,
        platform_token=platform_token,
        refresh_if_possible=refresh_if_possible,
        x_additional_headers=x_additional_headers,
        **kwargs,
    )
    if error:
        return None, error

    auto_refresh = kwargs.pop("auto_refresh", DEFAULT_AUTO_REFRESH)
    refresh_rate = kwargs.pop("refresh_rate", DEFAULT_REFRESH_RATE)
    if auto_refresh:
        set_on_demand_token_refresher(
            token_refresher=OnDemandTokenRefresher(
                refresher=LoginPlatformTimer(
                    0,
                    platform_id=platform_id,
                    platform_token=platform_token,
                    x_additional_headers=x_additional_headers,
                    **kwargs,
                ),
                refresh_rate=refresh_rate,
            ),
            **kwargs,
        )

    return token, None


@deprecated(
    message="2025-08-15 - This method is obsolete.",
    replacement="accelbyte_py_sdk.services.auth.v2.login_user",
)
def login_user(
    username: str,
    password: str,
    scope: Optional[Union[str, List[str]]] = None,
    refresh_if_possible: bool = False,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs,
):
    token, error = login_user_internal(
        username=username,
        password=password,
        scope=scope,
        refresh_if_possible=refresh_if_possible,
        x_additional_headers=x_additional_headers,
        **kwargs,
    )
    if error:
        return None, error

    auto_refresh = kwargs.pop("auto_refresh", DEFAULT_AUTO_REFRESH)
    refresh_rate = kwargs.pop("refresh_rate", DEFAULT_REFRESH_RATE)
    if auto_refresh:
        set_on_demand_token_refresher(
            token_refresher=OnDemandTokenRefresher(
                refresher=LoginUserTimer(
                    0,
                    username=username,
                    password=password,
                    scope=scope,
                    x_additional_headers=x_additional_headers,
                    **kwargs,
                ),
                refresh_rate=refresh_rate,
            ),
            **kwargs,
        )

    return token, None


@deprecated(
    message="2025-08-15 - This method is obsolete.",
    replacement="accelbyte_py_sdk.services.auth.v2.login_user_async",
)
async def login_user_async(
    username: str,
    password: str,
    scope: Optional[Union[str, List[str]]] = None,
    refresh_if_possible: bool = False,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs,
):
    token, error = await login_user_async_internal(
        username=username,
        password=password,
        scope=scope,
        refresh_if_possible=refresh_if_possible,
        x_additional_headers=x_additional_headers,
        **kwargs,
    )
    if error:
        return None, error

    auto_refresh = kwargs.pop("auto_refresh", DEFAULT_AUTO_REFRESH)
    refresh_rate = kwargs.pop("refresh_rate", DEFAULT_REFRESH_RATE)
    if auto_refresh:
        set_on_demand_token_refresher(
            token_refresher=OnDemandTokenRefresher(
                refresher=LoginUserTimer(
                    0,
                    username=username,
                    password=password,
                    scope=scope,
                    x_additional_headers=x_additional_headers,
                    **kwargs,
                ),
                refresh_rate=refresh_rate,
            ),
            **kwargs,
        )

    return token, None


@deprecated(
    message="2025-08-15 - This method is obsolete.",
    replacement="accelbyte_py_sdk.services.auth.v2.logout",
)
def logout(
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs,
):
    _, error = logout_internal(
        x_additional_headers=x_additional_headers,
        **kwargs,
    )
    if error:
        return None, error

    token_refresher = unset_on_demand_token_refresher(reset=True, **kwargs)

    return None, None


@deprecated(
    message="2025-08-15 - This method is obsolete.",
    replacement="accelbyte_py_sdk.services.auth.v2.logout_async",
)
async def logout_async(
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs,
):
    _, error = await logout_async_internal(
        x_additional_headers=x_additional_headers,
        **kwargs,
    )
    if error:
        return None, error

    token_refresher = unset_on_demand_token_refresher(reset=True, **kwargs)

    return None, None


@deprecated(
    message="2025-08-15 - This method is obsolete.",
    replacement="accelbyte_py_sdk.services.auth.v2.refresh_login",
)
def refresh_login(
    refresh_token: str, x_additional_headers: Optional[Dict[str, str]] = None, **kwargs
):
    return login_refresh_internal(
        refresh_token=refresh_token, x_additional_headers=x_additional_headers, **kwargs
    )


@deprecated(
    message="2025-08-15 - This method is obsolete.",
    replacement="accelbyte_py_sdk.services.auth.v2.refresh_login_async",
)
async def refresh_login_async(
    refresh_token: str, x_additional_headers: Optional[Dict[str, str]] = None, **kwargs
):
    return await login_refresh_async_internal(
        refresh_token=refresh_token, x_additional_headers=x_additional_headers, **kwargs
    )


@deprecated(
    message="2025-08-15 - This method is obsolete.",
    replacement="accelbyte_py_sdk.services.auth.v2.try_refresh_login",
)
def try_refresh_login(
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs,
) -> Optional[str]:
    return try_refresh_login_internal(
        x_additional_headers=x_additional_headers, **kwargs
    )


@deprecated(
    message="2025-08-15 - This method is obsolete.",
    replacement="accelbyte_py_sdk.services.auth.v2.try_refresh_login_async",
)
async def try_refresh_login_async(
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs,
) -> Optional[str]:
    return await try_refresh_login_async_internal(
        x_additional_headers=x_additional_headers, **kwargs
    )


login = login_user
login_async = login_user_async

# endregion v1


# region repeating


class LoginTimerBase(ABC, Timer):
    @abstractmethod
    def run(self):
        pass

    @abstractmethod
    async def run_async(self):
        pass


class LoginClientTimer(LoginTimerBase):
    def __init__(
        self,
        interval: Union[int, float],
        /,
        *,
        client_id: Optional[str] = None,
        client_secret: Optional[str] = None,
        x_additional_headers: Optional[Dict[str, str]] = None,
        refresh_rate: Optional[float] = None,
        repeats: Optional[int] = None,
        autostart: bool = False,
        repeat_on_exception: bool = DEFAULT_TIMER_REPEAT_ON_EXCEPTION,
        **kwargs,
    ) -> None:
        logger = kwargs.get("logger", AUTH_SERVICE_LOGGER)

        self.refresh_rate = refresh_rate
        self.sdk = kwargs.get("sdk", None)
        self.kwargs = {
            "client_id": client_id,
            "client_secret": client_secret,
            "x_additional_headers": x_additional_headers,
            **kwargs,
            **REFRESH_KWARGS,
        }

        super().__init__(
            interval,
            self.run,
            daemon=True,
            repeats=repeats,
            autostart=autostart,
            repeat_on_exception=repeat_on_exception,
            logger=logger,
        )

    def run(self):
        if self.refresh_rate is not None:
            sdk = self.sdk or SDK
            if sdk is not None:
                token_repository = sdk.get_token_repository(raise_when_none=False)
                if token_repository is not None:
                    needs_refresh = token_repository.has_token_expired(
                        multiplier=self.refresh_rate
                    )
                    if not needs_refresh:
                        token = token_repository.get_token()
                        return token, None

        return login_client(**self.kwargs)

    async def run_async(self):
        if self.refresh_rate is not None:
            sdk = self.sdk or SDK
            if sdk is not None:
                token_repository = sdk.get_token_repository(raise_when_none=False)
                if token_repository is not None:
                    needs_refresh = token_repository.has_token_expired(
                        multiplier=self.refresh_rate
                    )
                    if not needs_refresh:
                        token = token_repository.get_token()
                        return token, None

        return await login_client_async(**self.kwargs)


class LoginPlatformTimer(LoginTimerBase):
    def __init__(
        self,
        interval: Union[int, float],
        /,
        *,
        platform_id: str,
        platform_token: str,
        x_additional_headers: Optional[Dict[str, str]] = None,
        refresh_rate: Optional[float] = None,
        repeats: Optional[int] = None,
        autostart: bool = False,
        repeat_on_exception: bool = DEFAULT_TIMER_REPEAT_ON_EXCEPTION,
        **kwargs,
    ) -> None:
        logger = kwargs.get("logger", AUTH_SERVICE_LOGGER)

        self.refresh_rate = refresh_rate
        self.sdk = kwargs.get("sdk", None)
        self.kwargs = {
            "platform_id": platform_id,
            "platform_token": platform_token,
            "x_additional_headers": x_additional_headers,
            **kwargs,
            **REFRESH_KWARGS,
        }

        super().__init__(
            interval,
            self.run,
            daemon=True,
            repeats=repeats,
            autostart=autostart,
            repeat_on_exception=repeat_on_exception,
            logger=logger,
        )

    def run(self):
        if self.refresh_rate is not None:
            sdk = self.sdk or SDK
            if sdk is not None:
                token_repository = sdk.get_token_repository(raise_when_none=False)
                if token_repository is not None:
                    needs_refresh = token_repository.has_token_expired(
                        multiplier=self.refresh_rate
                    )
                    if not needs_refresh:
                        token = token_repository.get_token()
                        return token, None

        return login_platform(**self.kwargs)

    async def run_async(self):
        if self.refresh_rate is not None:
            sdk = self.sdk or SDK
            if sdk is not None:
                token_repository = sdk.get_token_repository(raise_when_none=False)
                if token_repository is not None:
                    needs_refresh = token_repository.has_token_expired(
                        multiplier=self.refresh_rate
                    )
                    if not needs_refresh:
                        token = token_repository.get_token()
                        return token, None

        return await login_platform_async(**self.kwargs)


class LoginUserTimer(LoginTimerBase):
    def __init__(
        self,
        interval: Union[int, float],
        /,
        *,
        username: str,
        password: str,
        scope: Optional[Union[str, List[str]]] = None,
        x_additional_headers: Optional[Dict[str, str]] = None,
        refresh_rate: Optional[float] = None,
        repeats: Optional[int] = None,
        autostart: bool = False,
        repeat_on_exception: bool = DEFAULT_TIMER_REPEAT_ON_EXCEPTION,
        **kwargs,
    ) -> None:
        logger = kwargs.get("logger", AUTH_SERVICE_LOGGER)

        self.refresh_rate = refresh_rate
        self.sdk = kwargs.get("sdk", None)
        self.kwargs = {
            "username": username,
            "password": password,
            "scope": scope,
            "x_additional_headers": x_additional_headers,
            **kwargs,
            **REFRESH_KWARGS,
        }

        super().__init__(
            interval,
            self.run,
            daemon=True,
            repeats=repeats,
            autostart=autostart,
            repeat_on_exception=repeat_on_exception,
            logger=logger,
        )

    def run(self):
        if self.refresh_rate is not None:
            sdk = self.sdk or SDK
            if sdk is not None:
                token_repository = sdk.get_token_repository(raise_when_none=False)
                if token_repository is not None:
                    needs_refresh = token_repository.has_token_expired(
                        multiplier=self.refresh_rate
                    )
                    if not needs_refresh:
                        token = token_repository.get_token()
                        return token, None

        return login_user(**self.kwargs)

    async def run_async(self):
        if self.refresh_rate is not None:
            sdk = self.sdk or SDK
            if sdk is not None:
                token_repository = sdk.get_token_repository(raise_when_none=False)
                if token_repository is not None:
                    needs_refresh = token_repository.has_token_expired(
                        multiplier=self.refresh_rate
                    )
                    if not needs_refresh:
                        token = token_repository.get_token()
                        return token, None

        return await login_user_async(**self.kwargs)


class RefreshLoginTimer(LoginTimerBase):
    def __init__(
        self,
        interval: Union[int, float],
        /,
        *,
        refresh_token: str,
        x_additional_headers: Optional[Dict[str, str]] = None,
        refresh_rate: Optional[float] = None,
        repeats: Optional[int] = None,
        autostart: bool = False,
        repeat_on_exception: bool = DEFAULT_TIMER_REPEAT_ON_EXCEPTION,
        **kwargs,
    ) -> None:
        logger = kwargs.get("logger", AUTH_SERVICE_LOGGER)

        self.refresh_rate = refresh_rate
        self.sdk = kwargs.get("sdk", None)
        self.kwargs = {
            "refresh_token": refresh_token,
            "x_additional_headers": x_additional_headers,
            **kwargs,
            **REFRESH_KWARGS,
        }

        super().__init__(
            interval,
            self.run,
            daemon=True,
            repeats=repeats,
            autostart=autostart,
            repeat_on_exception=repeat_on_exception,
            logger=logger,
        )

    def run(self):
        if self.refresh_rate is not None:
            sdk = self.sdk or SDK
            if sdk is not None:
                token_repository = sdk.get_token_repository(raise_when_none=False)
                if token_repository is not None:
                    needs_refresh = token_repository.has_token_expired(
                        multiplier=self.refresh_rate
                    )
                    if not needs_refresh:
                        token = token_repository.get_token()
                        return token, None

        return refresh_login(**self.kwargs)

    async def run_async(self):
        if self.refresh_rate is not None:
            sdk = self.sdk or SDK
            if sdk is not None:
                token_repository = sdk.get_token_repository(raise_when_none=False)
                if token_repository is not None:
                    needs_refresh = token_repository.has_token_expired(
                        multiplier=self.refresh_rate
                    )
                    if not needs_refresh:
                        token = token_repository.get_token()
                        return token, None

        return await refresh_login_async(**self.kwargs)


def set_on_demand_token_refresher(token_refresher: OnDemandTokenRefresher, **kwargs):
    key = "OnDemandTokenRefresher"
    sdk = kwargs.get("sdk", SDK)
    if existing := sdk.operation_preprocessors.pop(key, None):
        existing.reset()
    sdk.operation_preprocessors[key] = token_refresher


def unset_on_demand_token_refresher(reset: bool = True, **kwargs):
    key = "OnDemandTokenRefresher"
    sdk = kwargs.get("sdk", SDK)
    if existing := sdk.operation_preprocessors.pop(key, None):
        if not isinstance(existing, OnDemandTokenRefresher):
            raise TypeError(existing)
        if reset:
            existing.reset()
        return existing
    return None


class OnDemandTokenRefresher:
    def __init__(
        self, refresher: LoginTimerBase, refresh_rate: float = DEFAULT_REFRESH_RATE
    ):
        self._refresher = refresher
        self._refresh_rate = refresh_rate

        self._lock = threading.RLock()
        self._active_refresher: Optional[LoginTimerBase] = None

    def __call__(
        self, operation: Operation, sdk: AccelByteSDK, *args, **kwargs
    ) -> Tuple[Optional[Operation], Optional[HttpResponse]]:
        token_repo = sdk.get_token_repository(raise_when_none=False)
        if token_repo is None:
            return None, HttpResponse.create_token_repo_not_found_error()

        with self._lock:
            needs_refresh = token_repo.has_token() and token_repo.has_token_expired(
                multiplier=self._refresh_rate
            )
            if not needs_refresh:
                return operation, None
            if self._active_refresher is not None:
                if self._active_refresher.status == TimerStatus.RUNNING:
                    return operation, None
                elif self._active_refresher.status == TimerStatus.FINISHED:
                    self._active_refresher = None
                else:
                    raise NotImplementedError()
            self.refresh(token_repo=token_repo)

        return operation, None

    @property
    def status(self) -> TimerStatus:
        if self._active_refresher is None:
            return TimerStatus.IDLE
        return self._active_refresher.status

    def refresh(self, token_repo: TokenRepository):
        if refresh_token := token_repo.get_refresh_token():
            self._active_refresher = RefreshLoginTimer(
                0, refresh_token=refresh_token, autostart=True
            )
        else:
            self._active_refresher = self._refresher.clone(
                interval=0, repeats=None, autostart=True
            )

    def reset(self):
        if self._active_refresher is not None:
            self._active_refresher.cancel()


def enable_login_client_timer(
    sdk: AccelByteSDK,
    client_id: Optional[str] = None,
    client_secret: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    cancel_existing: bool = True,
    **kwargs,
) -> bool:
    interval: int = kwargs.pop("interval", DEFAULT_TIMER_INTERVAL)
    refresh_rate: float = kwargs.pop("refresh_rate", DEFAULT_TIMER_REFRESH_RATE)
    repeats: int = kwargs.pop("repeats", -1)
    autostart: bool = kwargs.pop("autostart", True)
    repeat_on_exception: bool = kwargs.pop(
        "repeat_on_exception", DEFAULT_TIMER_REPEAT_ON_EXCEPTION
    )

    if existing_timer := getattr(sdk, "timer", None):
        if not cancel_existing:
            return False
        cancel_method = getattr(existing_timer, "cancel", None)
        if callable(cancel_method):
            cancel_method()

    sdk.timer = LoginClientTimer(
        interval,
        client_id=client_id,
        client_secret=client_secret,
        x_additional_headers=x_additional_headers,
        refresh_rate=refresh_rate,
        repeats=repeats,
        autostart=autostart,
        repeat_on_exception=repeat_on_exception,
        sdk=sdk,
    )

    return True


def enable_login_platform_timer(
    sdk: AccelByteSDK,
    platform_id: str,
    platform_token: str,
    x_additional_headers: Optional[Dict[str, str]] = None,
    cancel_existing: bool = True,
    **kwargs,
) -> bool:
    interval: int = kwargs.pop("interval", DEFAULT_TIMER_INTERVAL)
    refresh_rate: float = kwargs.pop("refresh_rate", DEFAULT_TIMER_REFRESH_RATE)
    repeats: int = kwargs.pop("repeats", -1)
    autostart: bool = kwargs.pop("autostart", True)
    repeat_on_exception: bool = kwargs.pop(
        "repeat_on_exception", DEFAULT_TIMER_REPEAT_ON_EXCEPTION
    )

    if existing_timer := getattr(sdk, "timer", None):
        if not cancel_existing:
            return False
        cancel_method = getattr(existing_timer, "cancel", None)
        if callable(cancel_method):
            cancel_method()

    sdk.timer = LoginPlatformTimer(
        interval,
        platform_id=platform_id,
        platform_token=platform_token,
        x_additional_headers=x_additional_headers,
        refresh_rate=refresh_rate,
        repeats=repeats,
        autostart=autostart,
        repeat_on_exception=repeat_on_exception,
        sdk=sdk,
    )

    return True


def enable_login_user_timer(
    sdk: AccelByteSDK,
    username: str,
    password: str,
    scope: Optional[Union[str, List[str]]] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    cancel_existing: bool = True,
    **kwargs,
) -> bool:
    interval: int = kwargs.pop("interval", DEFAULT_TIMER_INTERVAL)
    refresh_rate: float = kwargs.pop("refresh_rate", DEFAULT_TIMER_REFRESH_RATE)
    repeats: int = kwargs.pop("repeats", -1)
    autostart: bool = kwargs.pop("autostart", True)
    repeat_on_exception: bool = kwargs.pop(
        "repeat_on_exception", DEFAULT_TIMER_REPEAT_ON_EXCEPTION
    )

    if existing_timer := getattr(sdk, "timer", None):
        if not cancel_existing:
            return False
        cancel_method = getattr(existing_timer, "cancel", None)
        if callable(cancel_method):
            cancel_method()

    sdk.timer = LoginUserTimer(
        interval,
        username=username,
        password=password,
        scope=scope,
        x_additional_headers=x_additional_headers,
        refresh_rate=refresh_rate,
        repeats=repeats,
        autostart=autostart,
        repeat_on_exception=repeat_on_exception,
        sdk=sdk,
    )

    return True


# endregion repeating



# Copyright (c) 2021 AccelByte Inc. All Rights Reserved.
# This is licensed software from AccelByte Inc, for limitations
# and restrictions contact your company contract manager.
#
# Code generated. DO NOT EDIT!

# template file: ags_py_codegen

# Custom Service Manager

# pylint: disable=duplicate-code
# pylint: disable=line-too-long
# pylint: disable=missing-function-docstring
# pylint: disable=missing-module-docstring
# pylint: disable=too-many-arguments
# pylint: disable=too-many-branches
# pylint: disable=too-many-instance-attributes
# pylint: disable=too-many-lines
# pylint: disable=too-many-locals
# pylint: disable=too-many-public-methods
# pylint: disable=too-many-return-statements
# pylint: disable=too-many-statements
# pylint: disable=unused-import

from __future__ import annotations
from typing import Any, Dict, List, Optional, Tuple, Union

from ....core import Model

from ..models.apimodel_app_status_progress_step import ApimodelAppStatusProgressStep


class ApimodelGetAppStatusProgressResponse(Model):
    """Apimodel get app status progress response (apimodel.GetAppStatusProgressResponse)

    Properties:
        app_id: (app_id) REQUIRED str

        app_name: (app_name) REQUIRED str

        operation_id: (operation_id) REQUIRED str

        steps: (steps) REQUIRED List[ApimodelAppStatusProgressStep]
    """

    # region fields

    app_id: str  # REQUIRED
    app_name: str  # REQUIRED
    operation_id: str  # REQUIRED
    steps: List[ApimodelAppStatusProgressStep]  # REQUIRED

    # endregion fields

    # region with_x methods

    def with_app_id(self, value: str) -> ApimodelGetAppStatusProgressResponse:
        self.app_id = value
        return self

    def with_app_name(self, value: str) -> ApimodelGetAppStatusProgressResponse:
        self.app_name = value
        return self

    def with_operation_id(self, value: str) -> ApimodelGetAppStatusProgressResponse:
        self.operation_id = value
        return self

    def with_steps(
        self, value: List[ApimodelAppStatusProgressStep]
    ) -> ApimodelGetAppStatusProgressResponse:
        self.steps = value
        return self

    # endregion with_x methods

    # region to methods

    def to_dict(self, include_empty: bool = False) -> dict:
        result: dict = {}
        if hasattr(self, "app_id"):
            result["app_id"] = str(self.app_id)
        elif include_empty:
            result["app_id"] = ""
        if hasattr(self, "app_name"):
            result["app_name"] = str(self.app_name)
        elif include_empty:
            result["app_name"] = ""
        if hasattr(self, "operation_id"):
            result["operation_id"] = str(self.operation_id)
        elif include_empty:
            result["operation_id"] = ""
        if hasattr(self, "steps"):
            result["steps"] = [
                i0.to_dict(include_empty=include_empty) for i0 in self.steps
            ]
        elif include_empty:
            result["steps"] = []
        return result

    # endregion to methods

    # region static methods

    @classmethod
    def create(
        cls,
        app_id: str,
        app_name: str,
        operation_id: str,
        steps: List[ApimodelAppStatusProgressStep],
        **kwargs,
    ) -> ApimodelGetAppStatusProgressResponse:
        instance = cls()
        instance.app_id = app_id
        instance.app_name = app_name
        instance.operation_id = operation_id
        instance.steps = steps
        return instance

    @classmethod
    def create_from_dict(
        cls, dict_: dict, include_empty: bool = False
    ) -> ApimodelGetAppStatusProgressResponse:
        instance = cls()
        if not dict_:
            return instance
        if "app_id" in dict_ and dict_["app_id"] is not None:
            instance.app_id = str(dict_["app_id"])
        elif include_empty:
            instance.app_id = ""
        if "app_name" in dict_ and dict_["app_name"] is not None:
            instance.app_name = str(dict_["app_name"])
        elif include_empty:
            instance.app_name = ""
        if "operation_id" in dict_ and dict_["operation_id"] is not None:
            instance.operation_id = str(dict_["operation_id"])
        elif include_empty:
            instance.operation_id = ""
        if "steps" in dict_ and dict_["steps"] is not None:
            instance.steps = [
                ApimodelAppStatusProgressStep.create_from_dict(
                    i0, include_empty=include_empty
                )
                for i0 in dict_["steps"]
            ]
        elif include_empty:
            instance.steps = []
        return instance

    @classmethod
    def create_many_from_dict(
        cls, dict_: dict, include_empty: bool = False
    ) -> Dict[str, ApimodelGetAppStatusProgressResponse]:
        return (
            {k: cls.create_from_dict(v, include_empty=include_empty) for k, v in dict_}
            if dict_
            else {}
        )

    @classmethod
    def create_many_from_list(
        cls, list_: list, include_empty: bool = False
    ) -> List[ApimodelGetAppStatusProgressResponse]:
        return (
            [cls.create_from_dict(i, include_empty=include_empty) for i in list_]
            if list_
            else []
        )

    @classmethod
    def create_from_any(
        cls, any_: any, include_empty: bool = False, many: bool = False
    ) -> Union[
        ApimodelGetAppStatusProgressResponse,
        List[ApimodelGetAppStatusProgressResponse],
        Dict[Any, ApimodelGetAppStatusProgressResponse],
    ]:
        if many:
            if isinstance(any_, dict):
                return cls.create_many_from_dict(any_, include_empty=include_empty)
            elif isinstance(any_, list):
                return cls.create_many_from_list(any_, include_empty=include_empty)
            else:
                raise ValueError()
        else:
            return cls.create_from_dict(any_, include_empty=include_empty)

    @staticmethod
    def get_field_info() -> Dict[str, str]:
        return {
            "app_id": "app_id",
            "app_name": "app_name",
            "operation_id": "operation_id",
            "steps": "steps",
        }

    @staticmethod
    def get_required_map() -> Dict[str, bool]:
        return {
            "app_id": True,
            "app_name": True,
            "operation_id": True,
            "steps": True,
        }

    # endregion static methods



# Copyright (c) 2021 AccelByte Inc. All Rights Reserved.
# This is licensed software from AccelByte Inc, for limitations
# and restrictions contact your company contract manager.
#
# Code generated. DO NOT EDIT!

# template file: ags_py_codegen

# AccelByte Gaming Services Session Service

# pylint: disable=duplicate-code
# pylint: disable=line-too-long
# pylint: disable=missing-function-docstring
# pylint: disable=missing-module-docstring
# pylint: disable=too-many-arguments
# pylint: disable=too-many-branches
# pylint: disable=too-many-instance-attributes
# pylint: disable=too-many-lines
# pylint: disable=too-many-locals
# pylint: disable=too-many-public-methods
# pylint: disable=too-many-return-statements
# pylint: disable=too-many-statements
# pylint: disable=unused-import

from __future__ import annotations
from typing import Any, Dict, List, Optional, Tuple, Union

from ....core import Model

from ..models.models_party_members import ModelsPartyMembers


class ModelsTeam(Model):
    """Models team (models.Team)

    Properties:
        parties: (parties) OPTIONAL List[ModelsPartyMembers]

        team_id: (teamID) OPTIONAL str

        user_i_ds: (userIDs) OPTIONAL List[str]
    """

    # region fields

    parties: List[ModelsPartyMembers]  # OPTIONAL
    team_id: str  # OPTIONAL
    user_i_ds: List[str]  # OPTIONAL

    # endregion fields

    # region with_x methods

    def with_parties(self, value: List[ModelsPartyMembers]) -> ModelsTeam:
        self.parties = value
        return self

    def with_team_id(self, value: str) -> ModelsTeam:
        self.team_id = value
        return self

    def with_user_i_ds(self, value: List[str]) -> ModelsTeam:
        self.user_i_ds = value
        return self

    # endregion with_x methods

    # region to methods

    def to_dict(self, include_empty: bool = False) -> dict:
        result: dict = {}
        if hasattr(self, "parties"):
            result["parties"] = [
                i0.to_dict(include_empty=include_empty) for i0 in self.parties
            ]
        elif include_empty:
            result["parties"] = []
        if hasattr(self, "team_id"):
            result["teamID"] = str(self.team_id)
        elif include_empty:
            result["teamID"] = ""
        if hasattr(self, "user_i_ds"):
            result["userIDs"] = [str(i0) for i0 in self.user_i_ds]
        elif include_empty:
            result["userIDs"] = []
        return result

    # endregion to methods

    # region static methods

    @classmethod
    def create(
        cls,
        parties: Optional[List[ModelsPartyMembers]] = None,
        team_id: Optional[str] = None,
        user_i_ds: Optional[List[str]] = None,
        **kwargs,
    ) -> ModelsTeam:
        instance = cls()
        if parties is not None:
            instance.parties = parties
        if team_id is not None:
            instance.team_id = team_id
        if user_i_ds is not None:
            instance.user_i_ds = user_i_ds
        return instance

    @classmethod
    def create_from_dict(cls, dict_: dict, include_empty: bool = False) -> ModelsTeam:
        instance = cls()
        if not dict_:
            return instance
        if "parties" in dict_ and dict_["parties"] is not None:
            instance.parties = [
                ModelsPartyMembers.create_from_dict(i0, include_empty=include_empty)
                for i0 in dict_["parties"]
            ]
        elif include_empty:
            instance.parties = []
        if "teamID" in dict_ and dict_["teamID"] is not None:
            instance.team_id = str(dict_["teamID"])
        elif include_empty:
            instance.team_id = ""
        if "userIDs" in dict_ and dict_["userIDs"] is not None:
            instance.user_i_ds = [str(i0) for i0 in dict_["userIDs"]]
        elif include_empty:
            instance.user_i_ds = []
        return instance

    @classmethod
    def create_many_from_dict(
        cls, dict_: dict, include_empty: bool = False
    ) -> Dict[str, ModelsTeam]:
        return (
            {k: cls.create_from_dict(v, include_empty=include_empty) for k, v in dict_}
            if dict_
            else {}
        )

    @classmethod
    def create_many_from_list(
        cls, list_: list, include_empty: bool = False
    ) -> List[ModelsTeam]:
        return (
            [cls.create_from_dict(i, include_empty=include_empty) for i in list_]
            if list_
            else []
        )

    @classmethod
    def create_from_any(
        cls, any_: any, include_empty: bool = False, many: bool = False
    ) -> Union[ModelsTeam, List[ModelsTeam], Dict[Any, ModelsTeam]]:
        if many:
            if isinstance(any_, dict):
                return cls.create_many_from_dict(any_, include_empty=include_empty)
            elif isinstance(any_, list):
                return cls.create_many_from_list(any_, include_empty=include_empty)
            else:
                raise ValueError()
        else:
            return cls.create_from_dict(any_, include_empty=include_empty)

    @staticmethod
    def get_field_info() -> Dict[str, str]:
        return {
            "parties": "parties",
            "teamID": "team_id",
            "userIDs": "user_i_ds",
        }

    @staticmethod
    def get_required_map() -> Dict[str, bool]:
        return {
            "parties": False,
            "teamID": False,
            "userIDs": False,
        }

    # endregion static methods



# Copyright (c) 2021 AccelByte Inc. All Rights Reserved.
# This is licensed software from AccelByte Inc, for limitations
# and restrictions contact your company contract manager.
#
# Code generated. DO NOT EDIT!

# template file: ags_py_codegen

# AccelByte Gaming Services Iam Service

# pylint: disable=duplicate-code
# pylint: disable=line-too-long
# pylint: disable=missing-function-docstring
# pylint: disable=missing-module-docstring
# pylint: disable=too-many-arguments
# pylint: disable=too-many-branches
# pylint: disable=too-many-instance-attributes
# pylint: disable=too-many-lines
# pylint: disable=too-many-locals
# pylint: disable=too-many-public-methods
# pylint: disable=too-many-return-statements
# pylint: disable=too-many-statements
# pylint: disable=unused-import

from __future__ import annotations
from typing import Any, Dict, List, Optional, Tuple, Union

from ....core import Model


class ModelPublicOpenIDUserInfoResponse(Model):
    """Model public open ID user info response (model.PublicOpenIDUserInfoResponse)

    Properties:
        email: (email) REQUIRED str

        email_verified: (email_verified) REQUIRED bool

        name: (name) REQUIRED str

        picture: (picture) REQUIRED str

        sub: (sub) REQUIRED str
    """

    # region fields

    email: str  # REQUIRED
    email_verified: bool  # REQUIRED
    name: str  # REQUIRED
    picture: str  # REQUIRED
    sub: str  # REQUIRED

    # endregion fields

    # region with_x methods

    def with_email(self, value: str) -> ModelPublicOpenIDUserInfoResponse:
        self.email = value
        return self

    def with_email_verified(self, value: bool) -> ModelPublicOpenIDUserInfoResponse:
        self.email_verified = value
        return self

    def with_name(self, value: str) -> ModelPublicOpenIDUserInfoResponse:
        self.name = value
        return self

    def with_picture(self, value: str) -> ModelPublicOpenIDUserInfoResponse:
        self.picture = value
        return self

    def with_sub(self, value: str) -> ModelPublicOpenIDUserInfoResponse:
        self.sub = value
        return self

    # endregion with_x methods

    # region to methods

    def to_dict(self, include_empty: bool = False) -> dict:
        result: dict = {}
        if hasattr(self, "email"):
            result["email"] = str(self.email)
        elif include_empty:
            result["email"] = ""
        if hasattr(self, "email_verified"):
            result["email_verified"] = bool(self.email_verified)
        elif include_empty:
            result["email_verified"] = False
        if hasattr(self, "name"):
            result["name"] = str(self.name)
        elif include_empty:
            result["name"] = ""
        if hasattr(self, "picture"):
            result["picture"] = str(self.picture)
        elif include_empty:
            result["picture"] = ""
        if hasattr(self, "sub"):
            result["sub"] = str(self.sub)
        elif include_empty:
            result["sub"] = ""
        return result

    # endregion to methods

    # region static methods

    @classmethod
    def create(
        cls,
        email: str,
        email_verified: bool,
        name: str,
        picture: str,
        sub: str,
        **kwargs,
    ) -> ModelPublicOpenIDUserInfoResponse:
        instance = cls()
        instance.email = email
        instance.email_verified = email_verified
        instance.name = name
        instance.picture = picture
        instance.sub = sub
        return instance

    @classmethod
    def create_from_dict(
        cls, dict_: dict, include_empty: bool = False
    ) -> ModelPublicOpenIDUserInfoResponse:
        instance = cls()
        if not dict_:
            return instance
        if "email" in dict_ and dict_["email"] is not None:
            instance.email = str(dict_["email"])
        elif include_empty:
            instance.email = ""
        if "email_verified" in dict_ and dict_["email_verified"] is not None:
            instance.email_verified = bool(dict_["email_verified"])
        elif include_empty:
            instance.email_verified = False
        if "name" in dict_ and dict_["name"] is not None:
            instance.name = str(dict_["name"])
        elif include_empty:
            instance.name = ""
        if "picture" in dict_ and dict_["picture"] is not None:
            instance.picture = str(dict_["picture"])
        elif include_empty:
            instance.picture = ""
        if "sub" in dict_ and dict_["sub"] is not None:
            instance.sub = str(dict_["sub"])
        elif include_empty:
            instance.sub = ""
        return instance

    @classmethod
    def create_many_from_dict(
        cls, dict_: dict, include_empty: bool = False
    ) -> Dict[str, ModelPublicOpenIDUserInfoResponse]:
        return (
            {k: cls.create_from_dict(v, include_empty=include_empty) for k, v in dict_}
            if dict_
            else {}
        )

    @classmethod
    def create_many_from_list(
        cls, list_: list, include_empty: bool = False
    ) -> List[ModelPublicOpenIDUserInfoResponse]:
        return (
            [cls.create_from_dict(i, include_empty=include_empty) for i in list_]
            if list_
            else []
        )

    @classmethod
    def create_from_any(
        cls, any_: any, include_empty: bool = False, many: bool = False
    ) -> Union[
        ModelPublicOpenIDUserInfoResponse,
        List[ModelPublicOpenIDUserInfoResponse],
        Dict[Any, ModelPublicOpenIDUserInfoResponse],
    ]:
        if many:
            if isinstance(any_, dict):
                return cls.create_many_from_dict(any_, include_empty=include_empty)
            elif isinstance(any_, list):
                return cls.create_many_from_list(any_, include_empty=include_empty)
            else:
                raise ValueError()
        else:
            return cls.create_from_dict(any_, include_empty=include_empty)

    @staticmethod
    def get_field_info() -> Dict[str, str]:
        return {
            "email": "email",
            "email_verified": "email_verified",
            "name": "name",
            "picture": "picture",
            "sub": "sub",
        }

    @staticmethod
    def get_required_map() -> Dict[str, bool]:
        return {
            "email": True,
            "email_verified": True,
            "name": True,
            "picture": True,
            "sub": True,
        }

    # endregion static methods
